@@ -130,6 +130,19 @@ class PySide2(QtDependency):
             return 'unknown version'
 
 
+class QtPy(Dependency):
+
+    @property
+    def installed(self):
+        try:
+            importlib.import_module(self.module)
+            return True
+        except Exception:
+            # QtPy raises a PythonQtError in some cases, so we can't use
+            # ImportError.
+            return False
+
+
 # Add any dependencies here
 # Make sure to add new categories to the categories tuple
 
@@ -143,13 +156,13 @@ gui_framework = (
 )
 
 required = (
-    Dependency('qtpy', 'Required', min_version='1.2'),
+    QtPy('qtpy', 'Required', min_version='1.2'),
     Dependency('setuptools', 'Required', min_version='1.0'),
     Dependency('numpy', 'Required', min_version='1.9'),
     Dependency('bottleneck', 'Required', min_version='1.2'),
     Dependency('matplotlib', 'Required for plotting', min_version='2.0'),
     Dependency('pandas', 'Adds support for Excel files and DataFrames', min_version='0.14'),
-    Dependency('astropy', 'Used for FITS I/O, table reading, and WCS Parsing', min_version='1.3'),
+    Dependency('astropy', 'Used for FITS I/O, table reading, and WCS Parsing', min_version='2.0'),
     Dependency('dill', 'Used when saving Glue sessions', min_version='0.2'),
     Dependency('h5py', 'Used to support HDF5 files', min_version='2.4'),
     Dependency('xlrd', 'Used to support Excel files', min_version='1.0'),
