@@ -1,11 +1,10 @@
 from __future__ import absolute_import, division, print_function
 
-from glue.external.qt import QtGui
+from qtpy import QtWidgets
 from glue.icons.qt import get_icon
-from glue.utils.qt.helpers import CUSTOM_QWIDGETS
 
 
-class GlueActionButton(QtGui.QPushButton):
+class GlueActionButton(QtWidgets.QPushButton):
 
     def set_action(self, action, text=True):
         self._text = text
@@ -22,12 +21,10 @@ class GlueActionButton(QtGui.QPushButton):
         self.setWhatsThis(self._action.whatsThis())
         self.setEnabled(self._action.isEnabled())
 
-CUSTOM_QWIDGETS.append(GlueActionButton)
-
 
 def action(name, parent, tip='', icon=None, shortcut=None):
     """ Factory for making a new action """
-    a = QtGui.QAction(name, parent)
+    a = QtWidgets.QAction(name, parent)
     a.setToolTip(tip)
     if icon:
         a.setIcon(get_icon(icon))
