@@ -4,7 +4,6 @@ import os
 import imp
 import sys
 from collections import namedtuple
-from glue.logger import logger
 
 """
 Objects used to configure Glue at runtime.
@@ -13,11 +12,15 @@ Objects used to configure Glue at runtime.
 __all__ = ['Registry', 'SettingRegistry', 'ExporterRegistry',
            'ColormapRegistry', 'DataFactoryRegistry', 'QtClientRegistry',
            'LinkFunctionRegistry', 'LinkHelperRegistry', 'ViewerToolRegistry',
-           'SingleSubsetLayerActionRegistry', 'ProfileFitterRegistry',
-           'qt_client', 'data_factory', 'link_function', 'link_helper',
-           'colormaps', 'exporters', 'settings', 'fit_plugin',
-           'auto_refresh', 'importer', 'DictRegistry', 'preference_panes',
-           'PreferencePanesRegistry']
+           'LayerActionRegistry', 'ProfileFitterRegistry', 'qt_client', 'data_factory',
+           'link_function', 'link_helper', 'colormaps', 'exporters', 'settings',
+           'fit_plugin', 'auto_refresh', 'importer', 'DictRegistry',
+           'preference_panes', 'PreferencePanesRegistry',
+           'DataExporterRegistry', 'data_exporter', 'layer_action',
+           'SubsetMaskExporterRegistry', 'SubsetMaskImporterRegistry',
+           'StartupActionRegistry', 'startup_action', 'QtFixedLayoutTabRegistry',
+           'qt_fixed_layout_tab', 'KeyboardShortcut', 'keyboard_shortcut',
+           'LayerArtistMakerRegistry', 'layer_artist_maker']
 
 
 CFG_DIR = os.path.join(os.path.expanduser('~'), '.glue')
@@ -178,11 +181,17 @@ class SettingRegistry(DictRegistry):
             value = self._members.get(key, self._defaults[key])
             yield key, value, self._validators[key]
 
+    def __str__(self):
+        s = ""
+        for name, value, validator in self:
+            s += "{0}: {1}\n".format(name, value)
+        return s
+
     def reset_defaults(self):
         self._members.clear()
 
     def is_default(self, setting):
-        return setting in self._defaults and not setting in self._members
+        return setting in self._defaults and setting not in self._members
 
 
 class QGlueParserRegistry(Registry):
@@ -194,7 +203,7 @@ class QGlueParserRegistry(Registry):
     with ``data_class``, ``parser`` and ``priority`` attributes, where ``class``
     defines the class for which to use the parser, and ``parser`` is a function
     that takes the input data and returns a list of glue
-    :class:`~glue.core.Data` objects. The ``parser`` functions should take two
+    :class:`~glue.core.data.Data` objects. The ``parser`` functions should take two
     arguments: the variable containing the data being parsed, and a label. In
     addition, the priority (defaulting to 0) can be specified in case one wants
     to make sure sub-classes get tested before more general classes. The
@@ -359,6 +368,10 @@ class ColormapRegistry(Registry):
         import matplotlib.cm as cm
         members = []
         members.append(['Gray', cm.gray])
+        members.append(['Viridis', cm.viridis])
+        members.append(['Plasma', cm.plasma])
+        members.append(['Inferno', cm.inferno])
+        members.append(['Magma', cm.magma])
         members.append(['Purple-Blue', cm.PuBu])
         members.append(['Yellow-Green-Blue', cm.YlGnBu])
         members.append(['Yellow-Orange-Red', cm.YlOrRd])
@@ -377,6 +390,18 @@ class ColormapRegistry(Registry):
         """
         self.members.append([label, cmap])
 
+    def __getitem__(self, cmap_name):
+        for name, cmap in self.members:
+            if name == cmap_name:
+                return cmap
+        raise KeyError(cmap_name)
+
+    def name_from_cmap(self, cmap_desired):
+        for name, cmap in self.members:
+            if cmap is cmap_desired:
+                return name
+        raise ValueError("Could not find name for colormap")
+
 
 class DataFactoryRegistry(Registry):
 
@@ -429,6 +454,41 @@ class DataFactoryRegistry(Registry):
             yield member
 
 
+class DataExporterRegistry(Registry):
+    """
+    Stores data exporters. Data exporters take a data/subset object as input
+    followed by a filename.
+    """
+
+    item = namedtuple('DataFactory', 'function label extension')
+
+    def __call__(self, label, extension=[]):
+        def adder(func):
+            self.add(self.item(func, label, extension))
+            return func
+        return adder
+
+    def __iter__(self):
+        for member in sorted(self.members, key=lambda x: x.label):
+            yield member
+
+
+class SubsetMaskExporterRegistry(DataExporterRegistry):
+    """
+    Stores mask exporters. Mask exporters should take a filename followed by
+    a dictionary of Numpy boolean arrays all with the same dimensions.
+    """
+    item = namedtuple('SubsetMaskExporter', 'function label extension')
+
+
+class SubsetMaskImporterRegistry(DataExporterRegistry):
+    """
+    Stores mask importers. Mask importers should take a filename and return a
+    dictionary of Numpy boolean arrays.
+    """
+    item = namedtuple('SubsetMaskImporter', 'function label extension')
+
+
 class QtClientRegistry(Registry):
     """
     Stores QT widgets to visualize data.
@@ -443,6 +503,18 @@ class QtClientRegistry(Registry):
     """
 
 
+class QtFixedLayoutTabRegistry(Registry):
+    """
+    Stores Qt pre-defined tabs (non-MDI)
+
+    New widgets can be registered via::
+
+        @qt_fixed_layout_tab
+        class CustomTab(QWidget):
+            ...
+    """
+
+
 class ViewerToolRegistry(DictRegistry):
 
     def add(self, tool_cls):
@@ -461,14 +533,45 @@ class ViewerToolRegistry(DictRegistry):
         return tool_cls
 
 
+class StartupActionRegistry(DictRegistry):
+
+    def add(self, startup_name, startup_function):
+        """
+        Add a startup function to the registry. This is a function that will
+        get called once glue has been started and any data loaded, and can
+        be used to set up specific layouts and create links.
+
+        Startup actions are triggered by either specifying comma-separated names
+        of actions on the command-line::
+
+            glue --startup=mystartupaction
+
+        or by passing an iterable of startup action names to the ``startup``
+        keyword of ``GlueApplication``.
+
+        The startup function will be given the session object and the data
+        collection object.
+        """
+        if startup_name in self.members:
+            raise ValueError("A startup action with the name '{0}' already exists".format(startup_name))
+        else:
+            self.members[startup_name] = startup_function
+
+    def __call__(self, name):
+        def adder(func):
+            self.add(name, func)
+            return func
+        return adder
+
+
 class LinkFunctionRegistry(Registry):
 
     """Stores functions to convert between quantities
 
-    The members properety is a list of (function, info_string,
+    The members property is a list of (function, info_string,
     output_labels) namedtuples. ``info_string`` describes what the
     function does. ``output_labels`` is a list of names for each output.
-    ``category`` is a category in which the link funtion will appear (defaults
+    ``category`` is a category in which the link function will appear (defaults
     to 'General').
 
     New link functions can be registered via
@@ -476,7 +579,7 @@ class LinkFunctionRegistry(Registry):
         @link_function(info="maps degrees to arcseconds",
                        output_labels=['arcsec'])
         def degrees2arcsec(degrees):
-            return degress * 3600
+            return degrees * 3600
 
     Link functions are expected to receive and return numpy arrays
     """
@@ -491,19 +594,44 @@ class LinkFunctionRegistry(Registry):
         return adder
 
 
-class SingleSubsetLayerActionRegistry(Registry):
-
-    """ Stores custom menu actions available when user selects a single
-        subset in the data collection view
-
-        This members property is a list of (label, tooltip, callback)
-        tuples. callback is a function that takes a Subset and DataCollection
-        as input
+class LayerActionRegistry(Registry):
     """
-    item = namedtuple('SingleSubsetLayerAction', 'label tooltip callback icon')
+    Stores custom menu actions available when the user select one or more
+    datasets, subset group, or subset in the data collection view.
 
-    def __call__(self, label, callback, tooltip=None, icon=None):
-        self.add(self.item(label, callback, tooltip, icon))
+    This members property is a list of named tuples with the following
+    attributes:
+
+    * ``label``: the user-facing name of the action
+    * ``tooltip``: the text that appears when hovering with the mouse over the action
+    * ``callback``: the function to call when the action is triggered
+    * ``icon``: an icon image to use for the layer action
+    * ``single``: whether to show this action only when selecting single layers (default: `False`)
+    * ``data``: if ``single`` is `True` whether to only show the action when selecting a dataset
+    * ``subset_group``: if ``single`` is `True` whether to only show the action when selecting a subset group
+    * ``subset``: if ``single`` is `True` whether to only show the action when selecting a subset
+
+    The callback function is called with two arguments. If ``single`` is
+    `True`, the first argument is the selected layer, otherwise it is the list
+    of selected layers. The second argument is the
+    `~glue.core.data_collection.DataCollection` object.
+    """
+    item = namedtuple('LayerAction', 'label tooltip callback icon single data subset_group, subset')
+
+    def __call__(self, label, callback=None, tooltip=None, icon=None, single=False,
+                 data=False, subset_group=False, subset=False):
+
+        # Backward-compatibility
+        if callback is not None:
+            self.add(self.item(label, tooltip, callback, icon, True,
+                               False, False, True))
+            return True
+
+        def adder(func):
+            self.add(self.item(label, tooltip, func, icon, single,
+                               data, subset_group, subset))
+            return func
+        return adder
 
 
 class LinkHelperRegistry(Registry):
@@ -513,7 +641,7 @@ class LinkHelperRegistry(Registry):
     The members property is a list of (object, info_string,
     input_labels) tuples. `Object` is the link helper. `info_string`
     describes what `object` does. `input_labels` is a list labeling
-    the inputs. ``category`` is a category in which the link funtion will appear
+    the inputs. ``category`` is a category in which the link function will appear
     (defaults to 'General').
 
     Each link helper takes a list of ComponentIDs as inputs, and
@@ -564,9 +692,97 @@ class BooleanSetting(object):
 
         return self.state
 
+
+class KeyboardShortcut(DictRegistry):
+    """
+    Stores keyboard shortcuts.
+    The members property is a dictionary within a dictionary of keyboard
+    shortcuts, which is represented as (viewer,(keybind,function)). The
+    ``function`` should take one item, which is a reference to the session.
+    """
+
+    def add(self, valid_viewers, keybind, function):
+        """
+        Add a new keyboard shortcut
+
+        Parameters
+        ----------
+        arg1: list
+            list of viewers where event can be fired
+        arg2: Qt.Key
+            type of key event
+        arg3: function()
+            function to be run that corresponds with key
+        """
+        if valid_viewers:
+            for viewer in valid_viewers:
+                if viewer in self.members:
+                    if keybind in self.members[viewer]:
+                        raise ValueError("Keyboard shortcut '{0}' already registered in {1}".format(keybind, viewer))
+                    else:
+                        self.members[viewer][keybind] = function
+                else:
+                    self.members[viewer] = {keybind: function}
+        else:
+            if None in self.members:
+                if keybind in self.members[None]:
+                    raise ValueError("Keyboard shortcut '{0}' already registered in {1}".format(keybind, None))
+                else:
+                    self.members[None][keybind] = function
+            else:
+                self.members[None] = {keybind: function}
+
+    def __call__(self, keybind, valid_viewers):
+        def adder(func):
+            self.add(valid_viewers, keybind, func)
+            return func
+        return adder
+
+
+class LayerArtistMakerRegistry(Registry):
+    """
+    A registry that allows customization of layer artists based on the data
+    and viewer type.
+    """
+
+    item = namedtuple('LayerArtistMaker', 'label function priority')
+
+    def add(self, label, function, priority=0):
+        """
+        Add a new plugin for providing custom layer artists.
+
+        Plugins take the form of functions that take two arguments - the data or
+        subset being added, and the viewer. The function should either return a
+        `~glue.viewers.common.layer_artist.LayerArtist` sub-class object or
+        `None`.
+
+        Parameters
+        ----------
+        label : str
+            Name for the plugin
+        function : callable
+            The function that returns layer artists
+        priority : int, optional
+            Set this to a higher number if multiple plugins are present
+            and you want your plugin to take precedence over another.
+        """
+        self.members.append(self.item(label, function, priority))
+
+    def __call__(self, label, priority=0):
+        def adder(func):
+            self.add(label, func, priority=priority)
+            return func
+        return adder
+
+    def __iter__(self):
+        for member in sorted(self.members, key=lambda x: -x.priority):
+            yield member
+
+
+layer_artist_maker = LayerArtistMakerRegistry()
 qt_client = QtClientRegistry()
+qt_fixed_layout_tab = QtFixedLayoutTabRegistry()
 viewer_tool = ViewerToolRegistry()
-data_factory = DataFactoryRegistry()
 link_function = LinkFunctionRegistry()
 link_helper = LinkHelperRegistry()
 colormaps = ColormapRegistry()
@@ -574,18 +790,30 @@ importer = DataImportRegistry()
 exporters = ExporterRegistry()
 settings = SettingRegistry()
 fit_plugin = ProfileFitterRegistry()
-single_subset_action = SingleSubsetLayerActionRegistry()
+layer_action = LayerActionRegistry()
 menubar_plugin = MenubarPluginRegistry()
 preference_panes = PreferencePanesRegistry()
 qglue_parser = QGlueParserRegistry()
+startup_action = StartupActionRegistry()
+keyboard_shortcut = KeyboardShortcut()
 
 # watch loaded data files for changes?
 auto_refresh = BooleanSetting(False)
 enable_contracts = BooleanSetting(False)
 
+# Data and subset I/O
+data_factory = DataFactoryRegistry()
+data_exporter = DataExporterRegistry()
+subset_mask_exporter = SubsetMaskExporterRegistry()
+subset_mask_importer = SubsetMaskImporterRegistry()
+
+# Backward-compatibility
+single_subset_action = layer_action
+
 
 def load_configuration(search_path=None):
-    ''' Find and import a config.py file
+    """
+    Find and import a config.py file
 
     Returns:
 
@@ -594,7 +822,7 @@ def load_configuration(search_path=None):
     Raises:
 
        Exception, if no module was found
-    '''
+    """
     search_order = search_path or _default_search_order()
     result = imp.new_module('config')
 
@@ -624,16 +852,14 @@ def _default_search_order():
        * Glue's own default config
     """
 
-    from glue import config
-
     search_order = [os.path.join(os.getcwd(), 'config.py')]
     if 'GLUERC' in os.environ:
         search_order.append(os.environ['GLUERC'])
-    search_order.append(os.path.join(config.CFG_DIR, 'config.py'))
+    search_order.append(os.path.join(CFG_DIR, 'config.py'))
     return search_order[::-1]
 
 
-###### Now define global settings ######
+# ##### Now define global settings ######
 
 GRAY = '#7F7F7F'
 BLUE = "#1F78B4"
@@ -651,8 +877,10 @@ LIGHT_ORANGE = "#FDBF6F"
 LIGHT_PURPLE = "#CAB2D6"
 
 settings.add('SUBSET_COLORS', [RED, GREEN, BLUE, BROWN, ORANGE, PURPLE, PINK], validator=list)
-settings.add('DATA_COLOR', '0.35')
+settings.add('DATA_COLOR', '#595959')
 settings.add('DATA_ALPHA', 0.8, validator=float)
 settings.add('BACKGROUND_COLOR', '#FFFFFF')
 settings.add('FOREGROUND_COLOR', '#000000')
 settings.add('SHOW_LARGE_DATA_WARNING', True, validator=bool)
+settings.add('SHOW_INFO_PROFILE_OPEN', True, validator=bool)
+settings.add('SHOW_WARN_PROFILE_DUPLICATE', True, validator=bool)
