@@ -2,7 +2,7 @@
 Glue's fitting classes are designed to be easily subclassed for performing
 custom model fitting in Glue.
 
-See the guide on :ref:`writing custom fit plugins <fit_plugins>` for
+See the guide on :ref:`writing custom fit plugins <custom-fitting>` for
 help with using custom fitting utilities in Glue.
 """
 
@@ -42,7 +42,7 @@ class BaseFitter1D(object):
             else:
                 setattr(self, k, v)
 
-    def plot(self, fit_result, axes, x):
+    def plot(self, fit_result, axes, x, linewidth=None, alpha=None, color=None, normalize=None):
         """
         Plot the result of a fit.
 
@@ -54,8 +54,10 @@ class BaseFitter1D(object):
                   plots will not be properly cleared if this isn't provided
         """
         y = self.predict(fit_result, x)
-        result = axes.plot(x, y, '#4daf4a',
-                           lw=3, alpha=0.8,
+        if normalize is not None:
+            y = normalize(y)
+        result = axes.plot(x, y, color,
+                           lw=linewidth, alpha=alpha,
                            scalex=False, scaley=False)
         return result
 
@@ -149,7 +151,7 @@ class BaseFitter1D(object):
         :type y:  :class:`numpy.ndarray`
         :param dy: 1 sigma uncertainties on each datum (optional)
         :type dy: :class:`numpy.ndarray`
-        :param constraints: The current value of :attr:`constraints`
+        :param constraints: The current value of the ``constraints`` property
         :param options: kwargs for model hyperparameters.
 
         :returns: An object representing the fit result.
@@ -159,7 +161,7 @@ class BaseFitter1D(object):
 
     def predict(self, fit_result, x):
         """
-        Evaulate the model at a set of locations.
+        Evaluate the model at a set of locations.
 
         **This must be overridden in a subclass.**
 
@@ -251,10 +253,10 @@ class AstropyFitter1D(BaseFitter1D):
         :type x: :class:`numpy.ndarray`
         :param y: Y - values of the data
         :type y: :class:`numpy.ndarray`
-        :param dy: ncertainties on Y(assumed to be 1 sigma)
+        :param dy: uncertainties on Y(assumed to be 1 sigma)
         :type dy: :class:`numpy.ndarray`
 
-        :returns: A dict maping ``{parameter_name: value guess}`` for each
+        :returns: A dict mapping ``{parameter_name: value guess}`` for each
                   parameter
         """
         return {}
@@ -330,7 +332,7 @@ try:
     class SimpleAstropyGaussianFitter(AstropyFitter1D):
 
         """
-        Guassian fitter using astropy.modeling.
+        Gaussian fitter using astropy.modeling.
         """
         model_cls = models.Gaussian1D
         try:
@@ -379,4 +381,5 @@ def _report_fitter(fitter):
         return "Converged in %i iterations" % fitter.fit_info['nfev']
     return 'Converged'
 
+
 __FITTERS__ = [PolynomialFitter, GaussianFitter]
