@@ -2,7 +2,7 @@
 LayerArtist classes handle the visualization of an individual subset
 or dataset.
 
-Visualization clients in Glue typically combose visualizations by stacking
+Visualization clients in Glue typically compose visualizations by stacking
 visualizations of several datasets and subsets on top of each other. They
 do this by creating and managing a collection of LayerArtists, one for
 each Data or Subset to view.
@@ -14,16 +14,17 @@ from __future__ import absolute_import, division, print_function
 
 import os
 from contextlib import contextmanager
-from abc import ABCMeta, abstractmethod
+from abc import ABCMeta
 
 import numpy as np
 
+from glue.external.echo.callback_container import CallbackContainer
 from glue.external import six
 from glue.core.subset import Subset
 from glue.utils import Pointer, PropertySetMixin
 from glue.core.message import LayerArtistEnabledMessage, LayerArtistDisabledMessage
 
-__all__ = ['LayerArtistBase', 'MatplotlibLayerArtist', 'LayerArtistContainer']
+__all__ = ['LayerArtistBase', 'LayerArtistContainer']
 
 
 DISABLED_LAYER_WARNING = """
@@ -179,32 +180,23 @@ class LayerArtistBase(PropertySetMixin):
     def layer(self, value):
         self._layer = value
 
-    @abstractmethod
     def redraw(self):
         """
         Re-render the plot
         """
-        raise NotImplementedError()
+        pass
 
-    @abstractmethod
-    def update(self, view=None):
+    def update(self):
         """
         Sync the visual appearance of the layer, and redraw
-
-        Subclasses may skip the update if the _changed attribute
-        is set to False.
-
-        Parameters
-        ----------
-        view : (ComponentID, numpy_style view) or None
-            A hint about what sub-view into the data is relevant.
         """
-        raise NotImplementedError()
+        pass
 
-    @abstractmethod
     def clear(self):
-        """Clear the visualization for this layer"""
-        raise NotImplementedError()
+        """
+        Clear the visualization for this layer
+        """
+        pass
 
     def remove(self):
         """
@@ -246,79 +238,14 @@ class LayerArtistBase(PropertySetMixin):
     __repr__ = __str__
 
 
-class MatplotlibLayerArtist(LayerArtistBase):
-
-    """
-    MPL-specific layer artist base class, that uses an Axes object
-    """
-
-    def __init__(self, layer, axes):
-        super(MatplotlibLayerArtist, self).__init__(layer)
-        self._axes = axes
-        self.artists = []
-
-    def redraw(self):
-        self._axes.figure.canvas.draw()
-
-    @property
-    def visible(self):
-        return self._visible
-
-    @visible.setter
-    def visible(self, value):
-        self._visible = value
-        for a in self.artists:
-            a.set_visible(value)
-
-    def _sync_style(self):
-        style = self.layer.style
-        for artist in self.artists:
-            edgecolor = style.color
-            # due to a bug in MPL 1.4.1, we can't disable the edge
-            # without making the whole point disappear. So we make the
-            # edge very thin instead
-            mew = 3 if style.marker == '+' else 0.01
-            artist.set_markeredgecolor(edgecolor)
-            artist.set_markeredgewidth(mew)
-            artist.set_markerfacecolor(style.color)
-            artist.set_marker(style.marker)
-            artist.set_markersize(style.markersize)
-            artist.set_linestyle('None')
-            artist.set_alpha(style.alpha)
-            artist.set_zorder(self.zorder)
-            artist.set_visible(self.visible and self.enabled)
-
-    @property
-    def zorder(self):
-        return self._zorder
-
-    @zorder.setter
-    def zorder(self, value):
-        for artist in self.artists:
-            artist.set_zorder(value)
-        self._zorder = value
-
-    @property
-    def enabled(self):
-        return len(self.artists) > 0
-
-    def clear(self):
-        for artist in self.artists:
-            try:
-                artist.remove()
-            except ValueError:  # already removed
-                pass
-        self.artists = []
-
-
 class LayerArtistContainer(object):
 
     """A collection of LayerArtists"""
 
     def __init__(self):
         self.artists = []
-        self.empty_callbacks = []
-        self.change_callbacks = []
+        self.empty_callbacks = CallbackContainer()
+        self.change_callbacks = CallbackContainer()
         self._ignore_callbacks = False
 
     def on_empty(self, func):
@@ -351,7 +278,7 @@ class LayerArtistContainer(object):
         """Remove a LayerArtist from this collection
 
         :param artist: The artist to remove
-        :type artist: :class:`MatplotlibLayerArtist`
+        :type artist: :class:`LayerArtistBase`
         """
         if artist in self.artists:
             self.artists.remove(artist)
@@ -373,12 +300,8 @@ class LayerArtistContainer(object):
         """
         Remove all callbacks
         """
-        if six.PY2:
-            self.empty_callbacks[:] = []
-            self.change_callbacks[:] = []
-        else:
-            self.empty_callbacks.clear()
-            self.change_callbacks.clear()
+        self.empty_callbacks.clear()
+        self.change_callbacks.clear()
 
     def _notify(self):
         if self._ignore_callbacks:
@@ -419,8 +342,6 @@ class LayerArtistContainer(object):
         return iter(sorted(self.artists, key=lambda x: x.zorder))
 
     def __contains__(self, item):
-        if isinstance(item, MatplotlibLayerArtist):
-            return item in self.artists
         return any(item is a.layer for a in self.artists)
 
     def __getitem__(self, layer):
