@@ -2,7 +2,7 @@
 This module provides several classes and LinkCollection classes to
 assist in linking data.
 
-The :class:`LinkCollection` class and its sublcasses are factories to create
+The :class:`LinkCollection` class and its sub-classes are factories to create
 multiple ComponentLinks easily. They are meant to be passed to
 :meth:`~glue.core.data_collection.DataCollection.add_link()`
 """
@@ -163,5 +163,5 @@ class LinkAligned(LinkCollection):
             if d.shape != shape:
                 raise TypeError("Input data do not have the same shape")
             for j in range(ndim):
-                self.extend(LinkSame(data[0].get_pixel_component_id(j),
-                                     data[i + 1].get_pixel_component_id(j)))
+                self.extend(LinkSame(data[0].pixel_component_ids[j],
+                                     data[i + 1].pixel_component_ids[j]))
