@@ -13,7 +13,7 @@ __all__ = ['Message', 'ErrorMessage', 'SubsetMessage', 'SubsetCreateMessage',
            'DataCollectionDeleteMessage', 'ApplicationClosedMessage',
            'DataRemoveComponentMessage', 'LayerArtistEnabledMessage',
            'LayerArtistDisabledMessage', 'DataRenameComponentMessage',
-           'DataReorderComponentMessage']
+           'DataReorderComponentMessage', 'LayerArtistVisibilityMessage']
 
 
 class Message(object):
@@ -80,7 +80,6 @@ class SubsetMessage(Message):
 
 
 class SubsetCreateMessage(SubsetMessage):
-
     """
     A message that a subset issues when its state changes
     """
@@ -121,8 +120,8 @@ class DataMessage(Message):
     """
 
     def __init__(self, sender, tag=None):
-        from glue.core.data import Data
-        if (not isinstance(sender, Data)):
+        from glue.core.data import BaseData
+        if (not isinstance(sender, BaseData)):
             raise TypeError("Sender must be a data instance: %s"
                             % type(sender))
         Message.__init__(self, sender, tag=tag)
@@ -252,7 +251,27 @@ class LayerArtistUpdatedMessage(Message):
         self.layer_artist = self.sender
 
 
+class LayerArtistVisibilityMessage(Message):
+    def __init__(self, sender, tag=None):
+        super(LayerArtistVisibilityMessage, self).__init__(sender, tag=tag)
+        self.layer_artist = self.sender
+
+
 class LayerArtistDisabledMessage(Message):
     def __init__(self, sender, tag=None):
         super(LayerArtistDisabledMessage, self).__init__(sender, tag=tag)
         self.layer_artist = self.sender
+
+
+class ComputationMessage(Message):
+    def __init__(self, sender, tag=None):
+        super(ComputationMessage, self).__init__(sender, tag=tag)
+        self.layer_artist = self.sender
+
+
+class ComputationStartedMessage(ComputationMessage):
+    pass
+
+
+class ComputationEndedMessage(ComputationMessage):
+    pass
