@@ -3,9 +3,10 @@ from __future__ import absolute_import, division, print_function
 import numpy as np
 
 from qtpy.QtCore import Qt
-from qtpy import QtCore, QtGui, QtWidgets
-from glue.core import roi
-from glue.utils.qt import mpl_to_qt4_color
+from qtpy import QtCore, QtGui
+
+from glue.core import roi as _roi
+from glue.utils.qt import mpl_to_qt_color
 
 
 class QtROI(object):
@@ -27,7 +28,13 @@ class QtROI(object):
 
     def _sync_patch(self):
         self.canvas.roi_callback = self._paint_check
-        self.canvas.update()  # QT repaint without MPL redraw
+        try:
+            self.canvas.update()  # QT repaint without MPL redraw
+        except RuntimeError:
+            # In some cases the above can raise a "wrapped C/C++ object of
+            # type ... has been deleted" error, in which case we can just
+            # ignore and carry on.
+            pass
 
     @property
     def canvas(self):
@@ -58,17 +65,31 @@ class QtROI(object):
     def _transform(self, x, y):
         """ Convert points from MPL data coords to Qt Widget coords"""
         t = self._axes.transData
+
         xy = np.column_stack((x, y))
         pts = t.transform(xy)
+
+        # Matplotlib 2.x with PyQt5 on a retina display has a bug which means
+        # that the coordinates returned by transData are twice as large as they
+        # should be. Since we don't know when/if this bug will be fixed, we
+        # check whether the coordinates of the top right corner are outside
+        # the canvas.
+        xmax = self._axes.get_xlim()[1]
+        ymax = self._axes.get_ylim()[1]
+        xd, yd = t.transform((xmax, ymax))
+        if xd > self.canvas.width() or yd > self.canvas.height():
+            ratio = self.canvas.devicePixelRatio()
+            pts /= ratio
+
         pts[:, 1] = self.canvas.height() - pts[:, 1]
         return pts[:, 0], pts[:, 1]
 
     def get_painter(self, canvas):
         p = QtGui.QPainter(canvas)
-        facecolor = mpl_to_qt4_color(self.plot_opts['facecolor'],
-                                     self.plot_opts['alpha'])
-        edgecolor = mpl_to_qt4_color(self.plot_opts['edgecolor'],
-                                     self.plot_opts['alpha'])
+        facecolor = mpl_to_qt_color(self.plot_opts['facecolor'],
+                                    self.plot_opts['alpha'])
+        edgecolor = mpl_to_qt_color(self.plot_opts['edgecolor'],
+                                    self.plot_opts['alpha'])
 
         pen = QtGui.QPen(edgecolor)
         pen.setWidth(self.plot_opts.get('edgewidth', 0))
@@ -79,7 +100,7 @@ class QtROI(object):
         return p
 
 
-class QtPathROI(QtROI, roi.MplPathROI):
+class QtPathROI(QtROI, _roi.MplPathROI):
 
     def get_painter(self, canvas):
         p = super(QtPathROI, self).get_painter(canvas)
@@ -99,22 +120,22 @@ class QtPathROI(QtROI, roi.MplPathROI):
         p.end()
 
 
-class QtRectangularROI(QtROI, roi.MplRectangularROI):
+class QtRectangularROI(QtROI, _roi.MplRectangularROI):
 
     def __init__(self, axes):
-        roi.MplRectangularROI.__init__(self, axes)
+        _roi.MplRectangularROI.__init__(self, axes)
 
 
-class QtPolygonalROI(QtROI, roi.MplPolygonalROI):
+class QtPolygonalROI(QtROI, _roi.MplPolygonalROI):
+
+    def __init__(self, axes, roi=None):
+        _roi.MplPolygonalROI.__init__(self, axes, roi=roi)
+
+
+class QtXRangeROI(QtROI, _roi.MplXRangeROI):
 
     def __init__(self, axes):
-        roi.MplPolygonalROI.__init__(self, axes)
-
-
-class QtXRangeROI(QtROI, roi.MplXRangeROI):
-
-    def __init__(self, axes):
-        roi.MplXRangeROI.__init__(self, axes)
+        _roi.MplXRangeROI.__init__(self, axes)
 
     def paint(self, canvas):
         x = self._roi.range()
@@ -125,10 +146,10 @@ class QtXRangeROI(QtROI, roi.MplXRangeROI):
                           [y[0], y[0], y[1], y[1]])
 
 
-class QtYRangeROI(QtROI, roi.MplYRangeROI):
+class QtYRangeROI(QtROI, _roi.MplYRangeROI):
 
     def __init__(self, axes):
-        roi.MplYRangeROI.__init__(self, axes)
+        _roi.MplYRangeROI.__init__(self, axes)
 
     def paint(self, canvas):
         y = self._roi.range()
@@ -139,14 +160,30 @@ class QtYRangeROI(QtROI, roi.MplYRangeROI):
                           [y[0], y[0], y[1], y[1]])
 
 
-class QtCircularROI(QtROI, roi.MplCircularROI):
+class QtCircularROI(QtROI, _roi.MplCircularROI):
 
     def __init__(self, axes):
-        roi.MplCircularROI.__init__(self, axes)
+        _roi.MplCircularROI.__init__(self, axes)
 
     def paint(self, canvas):
+
         xy = list(map(int, self._roi.get_center()))
         radius = int(self._roi.get_radius())
+
+        # Matplotlib 2.x with PyQt5 on a retina display has a bug which means
+        # that the coordinates returned by transData are twice as large as they
+        # should be. Since we don't know when/if this bug will be fixed, we
+        # check whether the coordinates of the top right corner are outside
+        # the canvas.
+        xmax = self._axes.get_xlim()[1]
+        ymax = self._axes.get_ylim()[1]
+        xd, yd = self._axes.transData.transform((xmax, ymax))
+        if xd > self.canvas.width() or yd > self.canvas.height():
+            ratio = self.canvas.devicePixelRatio()
+            xy[0] /= ratio
+            xy[1] /= ratio
+            radius /= ratio
+
         center = QtCore.QPoint(xy[0], canvas.height() - xy[1])
 
         p = self.get_painter(canvas)
