@@ -140,21 +140,7 @@ class StateAttributeCacheHelper(object):
 
     @property
     def data_values(self):
-        # For subsets in 'data' mode, we want to compute the limits based on
-        # the full dataset, not just the subset.
-        if isinstance(self.data, Subset):
-            return self.data.data[self.component_id]
-        else:
-            return self.data[self.component_id]
-
-    @property
-    def data_component(self):
-        # For subsets in 'data' mode, we want to compute the limits based on
-        # the full dataset, not just the subset.
-        if isinstance(self.data, Subset):
-            return self.data.data.get_component(self.component_id)
-        else:
-            return self.data.get_component(self.component_id)
+        return self.data[self.component_id]
 
     def invalidate_cache(self):
         self._cache.clear()
@@ -164,7 +150,12 @@ class StateAttributeCacheHelper(object):
         if self.attribute is None:
             return None
         else:
-            return self.attribute.parent
+            # For subsets in 'data' mode, we want to compute the limits based on
+            # the full dataset, not just the subset.
+            if isinstance(self.attribute.parent, Subset):
+                return self.attribute.parent.data
+            else:
+                return self.attribute.parent
 
     @property
     def component_id(self):
@@ -252,7 +243,7 @@ class StateAttributeLimitsHelper(StateAttributeCacheHelper):
     attribute : str
         The attribute name - this will be populated once a dataset is assigned
         to the helper.
-    percentile_subset : int
+    random_subset : int
         How many points to use at most for the percentile calculation (using all
         values is highly inefficient and not needed)
     margin : float
@@ -284,12 +275,12 @@ class StateAttributeLimitsHelper(StateAttributeCacheHelper):
     values_names = ('lower', 'upper')
     modifiers_names = ('log', 'percentile')
 
-    def __init__(self, state, attribute, percentile_subset=10000, margin=0, cache=None, **kwargs):
+    def __init__(self, state, attribute, random_subset=10000, margin=0, **kwargs):
 
-        super(StateAttributeLimitsHelper, self).__init__(state, attribute, cache=cache, **kwargs)
+        super(StateAttributeLimitsHelper, self).__init__(state, attribute, **kwargs)
 
         self.margin = margin
-        self.percentile_subset = percentile_subset
+        self.random_subset = random_subset
         self.subset_indices = None
 
         if self.attribute is not None:
@@ -314,8 +305,8 @@ class StateAttributeLimitsHelper(StateAttributeCacheHelper):
             percentile = 100
             log = False
         else:
-            percentile = self.percentile or 100
-            log = self.log or False
+            percentile = getattr(self, 'percentile', None) or 100
+            log = getattr(self, 'log', None) or False
 
         if not force and (percentile == 'Custom' or not hasattr(self, 'data') or self.data is None):
 
@@ -332,43 +323,26 @@ class StateAttributeLimitsHelper(StateAttributeCacheHelper):
 
             exclude = (100 - percentile) / 2.
 
-            data_values = self.data_values
+            if percentile == 100:
+                lower = self.data.compute_statistic('minimum', cid=self.component_id,
+                                                    finite=True, positive=log,
+                                                    random_subset=self.random_subset)
+                upper = self.data.compute_statistic('maximum', cid=self.component_id,
+                                                    finite=True, positive=log,
+                                                    random_subset=self.random_subset)
+            else:
+                lower = self.data.compute_statistic('percentile', cid=self.component_id,
+                                                    percentile=exclude, positive=log,
+                                                    random_subset=self.random_subset)
+                upper = self.data.compute_statistic('percentile', cid=self.component_id,
+                                                    percentile=100 - exclude, positive=log,
+                                                    random_subset=self.random_subset)
 
-            if data_values.size > self.percentile_subset:
-                if self.subset_indices is None or self.subset_indices[0] != data_values.size:
-                    self.subset_indices = (data_values.size,
-                                           np.random.randint(0, data_values.size,
-                                                             self.percentile_subset))
-                data_values = data_values.ravel()[self.subset_indices[1]]
+            if not isinstance(lower, np.datetime64) and np.isnan(lower):
+                lower, upper = 0, 1
+            else:
 
-            if log:
-                data_values = data_values[data_values > 0]
-                if len(data_values) == 0:
-                    self.set(lower=0.1, upper=1, percentile=percentile, log=log)
-                    return
-
-            # NOTE: we can't use np.nanmin/np.nanmax or nanpercentile below as
-            # they don't exclude inf/-inf
-            if data_values.dtype.kind != 'M':
-                data_values = data_values[np.isfinite(data_values)]
-
-            if data_values.size > 0:
-
-                if percentile == 100:
-
-                    if data_values.dtype.kind == 'M':
-                        lower = data_values.min()
-                        upper = data_values.max()
-                    else:
-                        lower = np.min(data_values)
-                        upper = np.max(data_values)
-
-                else:
-
-                    lower = np.percentile(data_values, exclude)
-                    upper = np.percentile(data_values, 100 - exclude)
-
-                if self.data_component.categorical:
+                if self.data.get_kind(self.component_id) == 'categorical':
                     lower = np.floor(lower - 0.5) + 0.5
                     upper = np.ceil(upper + 0.5) - 0.5
 
@@ -381,11 +355,6 @@ class StateAttributeLimitsHelper(StateAttributeCacheHelper):
                     lower -= value_range * self.margin
                     upper += value_range * self.margin
 
-            else:
-
-                lower = 0.
-                upper = 1.
-
             self.set(lower=lower, upper=upper, percentile=percentile, log=log)
 
     def flip_limits(self):
@@ -414,7 +383,7 @@ class StateAttributeSingleValueHelper(StateAttributeCacheHelper):
             if self.mode == 'values':
                 arg = self.data_values
             else:
-                arg = self.data_component
+                arg = self.data.get_component(self.component_id)
             self.set(value=self._function(arg))
 
 
@@ -423,14 +392,17 @@ class StateAttributeHistogramHelper(StateAttributeCacheHelper):
     values_names = ('lower', 'upper', 'n_bin')
     modifiers_names = ()
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, state, attribute, random_subset=10000, max_n_bin=30,
+                 default_n_bin=15, common_n_bin=None, **kwargs):
 
-        self._max_n_bin = kwargs.pop('max_n_bin', 30)
-        self._default_n_bin = kwargs.pop('default_n_bin', 15)
+        self._max_n_bin = max_n_bin
+        self._default_n_bin = default_n_bin
 
-        common_n_bin_att = kwargs.pop('common_n_bin', None)
+        common_n_bin_att = common_n_bin
 
-        super(StateAttributeHistogramHelper, self).__init__(*args, **kwargs)
+        super(StateAttributeHistogramHelper, self).__init__(state, attribute, **kwargs)
+
+        self.random_subset = random_subset
 
         if common_n_bin_att is not None:
             if getattr(self._state, common_n_bin_att):
@@ -443,13 +415,12 @@ class StateAttributeHistogramHelper(StateAttributeCacheHelper):
 
     def _apply_common_n_bin(self):
         for att in self._cache:
-            cmp = self.data.get_component(att)
-            if not cmp.categorical:
+            if not self.data.get_kind(att) == 'categorical':
                 self._cache[att]['n_bin'] = self._common_n_bin
 
     def _update_common_n_bin(self, common_n_bin):
         if common_n_bin:
-            if self.data_component.categorical:
+            if self.data.get_kind(self.component_id) == 'categorical':
                 self._common_n_bin = self._default_n_bin
             else:
                 self._common_n_bin = self.n_bin
@@ -463,21 +434,23 @@ class StateAttributeHistogramHelper(StateAttributeCacheHelper):
             self.set()
             return
 
-        comp = self.data_component
+        categorical = self.data.get_kind(self.component_id) == 'categorical'
 
         if 'n_bin' in properties:
             self.set()
-            if self._common_n_bin is not None and not comp.categorical:
+            if self._common_n_bin is not None and not categorical:
                 self._common_n_bin = properties['n_bin']
                 self._apply_common_n_bin()
 
         if 'attribute' in properties or force:
 
-            if comp.categorical:
+            if categorical:
 
-                n_bin = max(1, min(comp.categories.size, self._max_n_bin))
+                n_categories = self.data_values.categories.size
+
+                n_bin = max(1, min(n_categories, self._max_n_bin))
                 lower = -0.5
-                upper = lower + comp.categories.size
+                upper = lower + n_categories
 
             else:
 
@@ -486,19 +459,13 @@ class StateAttributeHistogramHelper(StateAttributeCacheHelper):
                 else:
                     n_bin = self._common_n_bin
 
-                values = self.data_values
+                lower = self.data.compute_statistic('minimum', cid=self.component_id,
+                                                    finite=True, random_subset=self.random_subset)
+                upper = self.data.compute_statistic('maximum', cid=self.component_id,
+                                                    finite=True, random_subset=self.random_subset)
 
-                # NOTE: we can't use np.nanmin/np.nanmax or nanpercentile below as
-                # they don't exclude inf/-inf
-                if values.dtype.kind != 'M':
-                    values = values[np.isfinite(values)]
-
-                if values.size > 0:
-                    lower = values.min()
-                    upper = values.max()
-                else:
-                    lower = 0.
-                    upper = 1.
+                if not isinstance(lower, np.datetime64) and np.isnan(lower):
+                    lower, upper = 0, 1
 
             self.set(lower=lower, upper=upper, n_bin=n_bin)
 
