@@ -113,8 +113,7 @@ class TestCommandStack(object):
         cmd = c.NewDataViewer(viewer=None, data=None)
         v = self.stack.do(cmd)
 
-        self.session.application.new_data_viewer.assert_called_once_with(
-            None, None)
+        self.session.application.new_data_viewer.assert_called_once_with(None, None)
 
         self.stack.undo()
         v.close.assert_called_once_with(warn=False)
@@ -126,13 +125,13 @@ class TestCommandStack(object):
         dc.append(x)
 
         r = MagicMock(roi.Roi)
-        client = MagicMock(core.client.Client)
-        client.data = dc
+        apply_roi = MagicMock()
 
-        cmd = c.ApplyROI(client=client, roi=r)
+        cmd = c.ApplyROI(data_collection=dc, roi=r,
+                         apply_func=apply_roi)
 
         self.stack.do(cmd)
-        client.apply_roi.assert_called_once_with(r)
+        apply_roi.assert_called_once_with(r)
 
         old_state = s.subset_state
         s.subset_state = MagicMock(spec_set=core.subset.SubsetState)
