@@ -4,8 +4,7 @@ from __future__ import absolute_import, division, print_function
 
 import pytest
 import numpy as np
-from mock import patch
-from numpy.testing import assert_allclose
+from numpy.testing import assert_allclose, assert_equal
 
 from glue.tests.helpers import requires_astropy
 
@@ -134,16 +133,12 @@ class TestWcsCoordinates(object):
         assert_allclose(result[1], expected[1])
 
     def test_pixel2world_invalid_input(self):
-        coord = WCSCoordinates(None)
-        x, y = {}, {}
-        with pytest.raises(TypeError) as exc:
-            coord.pixel2world(x, y)
+        with pytest.raises(ValueError):
+            WCSCoordinates(None)
 
     def test_world2pixel_invalid_input(self):
-        coord = WCSCoordinates(None)
-        x, y = {}, {}
-        with pytest.raises(TypeError) as exc:
-            coord.world2pixel(x, y)
+        with pytest.raises(ValueError):
+            WCSCoordinates(None)
 
     def test_axis_label(self):
         hdr = self.default_header()
@@ -204,6 +199,7 @@ class TestCoordinatesFromHeader(object):
         coord = coordinates_from_header(hdr)
         assert type(coord) == Coordinates
 
+
 HDR_2D_VALID = """
 SIMPLE  =                    T / Written by IDL:  Wed Jul 27 10:01:47 2011
 BITPIX  =                  -32 / number of bits per data pixel
@@ -322,8 +318,55 @@ def test_coords_preserve_shape_3d():
     for r in result:
         assert r.shape == x.shape
 
+
 def test_world_axis_unit():
     coord = coordinates_from_header(header_from_string(HDR_3D_VALID_WCS))
     assert coord.world_axis_unit(0) == 'm / s'
     assert coord.world_axis_unit(1) == 'deg'
     assert coord.world_axis_unit(2) == 'deg'
+
+
+def test_dependent_axes_non_diagonal_pc():
+
+    # Fix a bug that occurred when non-diagonal PC elements
+    # were present in the WCS - in that case all other axes
+    # were returned as dependent axes even if this wasn't
+    # the case.
+
+    from astropy.wcs import WCS
+
+    wcs = WCS(naxis=3)
+    wcs.wcs.ctype = 'HPLN-TAN', 'HPLT-TAN', 'Time'
+    wcs.wcs.crval = 1, 1, 1
+    wcs.wcs.crpix = 1, 1, 1
+    wcs.wcs.cd = [[0.9, 0.1, 0], [-0.1, 0.9, 0], [0, 0, 1]]
+
+    # Remember that the axes numbers below are reversed compared
+    # to the WCS order above.
+    coord = WCSCoordinates(wcs=wcs)
+    assert_equal(coord.dependent_axes(0), [0])
+    assert_equal(coord.dependent_axes(1), [1, 2])
+    assert_equal(coord.dependent_axes(2), [1, 2])
+
+
+def test_pixel2world_single_axis():
+
+    # Regression test for a bug in pixel2world_single_axis which was due to
+    # incorrect indexing order (WCS vs Numpy)
+
+    from astropy.wcs import WCS
+
+    wcs = WCS(naxis=3)
+    wcs.wcs.ctype = 'HPLN-TAN', 'HPLT-TAN', 'Time'
+    wcs.wcs.crval = 1, 1, 1
+    wcs.wcs.crpix = 1, 1, 1
+    wcs.wcs.cd = [[0.9, 0.1, 0], [-0.1, 0.9, 0], [0, 0, 1]]
+
+    x = np.array([0.2, 0.4, 0.6])
+    y = np.array([0.3, 0.6, 0.9])
+    z = np.array([0.5, 0.5, 0.5])
+
+    coord = WCSCoordinates(wcs=wcs)
+    assert_allclose(coord.pixel2world_single_axis(x, y, z, axis=0), [1.21004705, 1.42012044, 1.63021455])
+    assert_allclose(coord.pixel2world_single_axis(x, y, z, axis=1), [1.24999002, 1.499947, 1.74985138])
+    assert_allclose(coord.pixel2world_single_axis(x, y, z, axis=2), [1.5, 1.5, 1.5])
