@@ -4,11 +4,13 @@ from __future__ import absolute_import, division, print_function
 
 import pytest
 import numpy as np
-from numpy.testing import assert_equal
+from numpy.testing import assert_equal, assert_allclose
 from mock import MagicMock
+from astropy.utils import NumpyRNGContext
 
 from glue.external import six
 from glue import core
+from glue.utils import broadcast_to
 
 from ..component import Component, DerivedComponent, CategoricalComponent, DateTimeComponent
 from ..component_id import ComponentID
@@ -21,9 +23,9 @@ from ..exceptions import IncompatibleAttribute
 from ..hub import Hub
 from ..registry import Registry
 from ..subset import (Subset, CategoricalROISubsetState, SubsetState,
-                      RoiSubsetState, RangeSubsetState,
-                      CategoricalMultiRangeSubsetState,
-                      CategoricalROISubsetState2D, AndState)
+                      RoiSubsetState, RangeSubsetState, SliceSubsetState,
+                      CategoricalMultiRangeSubsetState, MaskSubsetState,
+                      CategoricalROISubsetState2D, AndState, roi_to_subset_state)
 from ..roi import PolygonalROI, CategoricalROI, RangeROI, RectangularROI
 
 from .test_state import clone
@@ -140,7 +142,7 @@ class TestData(object):
         labels = 'asldfkjaAREGWoibasiwnsldkgajsldkgslkg'
         for label in labels:
             data.add_component(comp, label)
-        ids = data.visible_components
+        ids = data.main_components
         assert [cid.label for cid in ids] == list(labels)
 
     def test_broadcast(self):
@@ -163,19 +165,16 @@ class TestData(object):
         assert exc.value.args[0] == ("Data has already been assigned "
                                      "to a different hub")
 
-    def test_primary_components(self):
+    def test_main_components(self):
         compid = ComponentID('virtual')
         link = MagicMock(spec_set=ComponentLink)
         comp = DerivedComponent(self.data, link)
 
         self.data.add_component(comp, compid)
 
-        pricomps = self.data.primary_components
-        print(self.comp_id, compid, pricomps)
-        print(self.comp_id in pricomps)
-        print(compid not in pricomps)
-        assert self.comp_id in pricomps
-        assert compid not in pricomps
+        main_comps = self.data.main_components
+        assert self.comp_id in main_comps
+        assert compid not in main_comps
 
     def test_add_component_invalid_component(self):
         comp = Component(np.array([1]))
@@ -283,10 +282,10 @@ class TestData(object):
 
     def test_coordinate_links(self):
         links = self.data.coordinate_links
-        w0 = self.data[self.data.get_world_component_id(0)]
-        w1 = self.data[self.data.get_world_component_id(1)]
-        p0 = self.data[self.data.get_pixel_component_id(0)]
-        p1 = self.data[self.data.get_pixel_component_id(1)]
+        w0 = self.data[self.data.world_component_ids[0]]
+        w1 = self.data[self.data.world_component_ids[1]]
+        p0 = self.data[self.data.pixel_component_ids[0]]
+        p1 = self.data[self.data.pixel_component_ids[1]]
 
         w0prime = links[0].compute(self.data)
         p0prime = links[1].compute(self.data)
@@ -330,6 +329,7 @@ class TestData(object):
         assert 'read-only' in exc.value.args[0]
         assert not d['x'].flags['WRITEABLE']
 
+    @pytest.mark.xfail
     def test_categorical_immutable(self):
         d = Data()
         c = CategoricalComponent(['M', 'M', 'F'], categories=['M', 'F'])
@@ -433,14 +433,15 @@ class TestROICreation(object):
         d = Data(xdata=[1, 2, 3], ydata=[1, 2, 3])
         comp = d.get_component(d.id['xdata'])
         roi = RangeROI('x', min=2, max=3)
-        s = comp.subset_from_roi('xdata', roi)
+        s = roi_to_subset_state(roi, x_att='xdata')
         assert isinstance(s, RangeSubsetState)
         np.testing.assert_array_equal((s.lo, s.hi),
                                       [2, 3])
 
         roi = RangeROI('y', min=2, max=3)
-        s = comp.subset_from_roi('xdata', roi, other_att='ydata',
-                                 other_comp=d.get_component(d.id['ydata']))
+        s = roi_to_subset_state(roi,
+                                x_att='xdata',
+                                y_att='ydata')
         assert isinstance(s, RangeSubsetState)
         assert s.att == 'ydata'
 
@@ -449,13 +450,13 @@ class TestROICreation(object):
         d = Data(x=['a', 'b', 'c'], y=[1, 2, 3])
         comp = d.get_component(d.id['x'])
         roi = CategoricalROI(['b', 'c'])
-        s = comp.subset_from_roi(d.id['x'], roi)
+        s = roi_to_subset_state(roi, x_att=d.id['x'], x_categories=comp.categories)
         assert isinstance(s, CategoricalROISubsetState)
         np.testing.assert_array_equal((s.roi.contains(['a', 'b', 'c'], None)),
                                       [False, True, True])
 
         roi = RangeROI('x', min=1, max=3)
-        s = comp.subset_from_roi('x', roi)
+        s = roi_to_subset_state(roi, x_att='x', x_categories=comp.categories)
         assert isinstance(s, CategoricalROISubsetState)
         np.testing.assert_array_equal((s.roi.contains(['a', 'b', 'c'], None)),
                                       [False, True, True])
@@ -463,20 +464,17 @@ class TestROICreation(object):
     def test_polygon_roi(self):
 
         d = Data(x=[1, 1.3, 3, 10], y=[1, 1.5, 3, 10])
-        x_comp = d.get_component(d.id['x'])
-        y_comp = d.get_component(d.id['y'])
         roi = PolygonalROI([0, 0, 2, 2], [0, 2, 2, 0])
-        s = x_comp.subset_from_roi(d.id['x'], roi, other_comp=y_comp, other_att=d.id['y'])
+        s = roi_to_subset_state(roi, x_att=d.id['x'], y_att=d.id['y'])
         assert isinstance(s, RoiSubsetState)
         np.testing.assert_array_equal(s.to_mask(d), [True, True, False, False])
 
     def test_polygon_categorical_rectangular(self):
 
         d = Data(x=[1, 1.3, 3, 10], y=['a', 'b', 'c', 'd'])
-        x_comp = d.get_component(d.id['x'])
         y_comp = d.get_component(d.id['y'])
         roi = PolygonalROI([0, 0, 2, 2], [0, 2, 2, 0])
-        s = x_comp.subset_from_roi('x', roi, other_comp=y_comp, other_att='y')
+        s = roi_to_subset_state(roi, x_att='x', y_att='y', y_categories=y_comp.categories)
         assert isinstance(s, CategoricalMultiRangeSubsetState)
 
         np.testing.assert_array_equal(s.to_mask(d), [True, True, False, False])
@@ -484,10 +482,9 @@ class TestROICreation(object):
     def test_polygon_categorical_arbitrary(self):
 
         d = Data(x=[1, 1.3, 3, 10], y=['a', 'b', 'c', 'd'])
-        x_comp = d.get_component(d.id['x'])
         y_comp = d.get_component(d.id['y'])
         roi = PolygonalROI([0, 4, 4, 1, 0], [-0.5, 3.5, 0, -1, -0.5])
-        s = x_comp.subset_from_roi('x', roi, other_comp=y_comp, other_att='y')
+        s = roi_to_subset_state(roi, x_att='x', y_att='y', y_categories=y_comp.categories)
         assert isinstance(s, CategoricalMultiRangeSubsetState)
 
         np.testing.assert_array_equal(s.to_mask(d), [True, False, True, False])
@@ -498,12 +495,13 @@ class TestROICreation(object):
         x_comp = d.get_component(d.id['x'])
         y_comp = d.get_component(d.id['y'])
         roi = RectangularROI(xmin=-0.1, xmax=2.1, ymin=-0.1, ymax=2.1)
-        s = x_comp.subset_from_roi('x', roi, other_comp=y_comp, other_att='y')
+
+        s = roi_to_subset_state(roi, x_att='x', y_att='y', y_categories=y_comp.categories)
         assert isinstance(s, AndState)
 
         np.testing.assert_array_equal(s.to_mask(d), [True, True, False, False])
 
-        s = y_comp.subset_from_roi('y', roi, other_comp=x_comp, other_att='x')
+        s = roi_to_subset_state(roi, x_att='x', y_att='y', y_categories=y_comp.categories)
         assert isinstance(s, AndState)
 
         np.testing.assert_array_equal(s.to_mask(d), [True, True, False, False])
@@ -514,7 +512,9 @@ class TestROICreation(object):
         x_comp = d.get_component(d.id['x'])
         y_comp = d.get_component(d.id['y'])
         roi = PolygonalROI([0.5, 1.5, 2.5, 1, 0.5], [0.5, 0.5, 2.5, 3.5, 0.5])
-        s = x_comp.subset_from_roi('x', roi, other_comp=y_comp, other_att='y')
+        s = roi_to_subset_state(roi,
+                                x_att='x', x_categories=x_comp.categories,
+                                y_att='y', y_categories=y_comp.categories)
         assert isinstance(s, CategoricalROISubsetState2D)
 
         np.testing.assert_array_equal(s.to_mask(d), [False, True, True, False, True, False])
@@ -525,7 +525,9 @@ class TestROICreation(object):
         x_comp = d.get_component(d.id['x'])
         y_comp = d.get_component(d.id['y'])
         roi = PolygonalROI([0.5, 0.6, 0.6, 0.5], [0.5, 0.5, 0.6, 0.5])
-        s = x_comp.subset_from_roi('x', roi, other_comp=y_comp, other_att='y')
+        s = roi_to_subset_state(roi,
+                                x_att='x', x_categories=x_comp.categories,
+                                y_att='y', y_categories=y_comp.categories)
         assert isinstance(s, CategoricalROISubsetState2D)
 
         np.testing.assert_array_equal(s.to_mask(d), [False, False, False, False, False, False])
@@ -612,11 +614,11 @@ def test_foreign_pixel_components_not_in_visible():
     dc = DataCollection([d1, d2])
     dc.add_link(LinkSame(d1.id['x'], d2.id['w']))
 
-    dc.add_link(LinkSame(d1.get_world_component_id(0),
-                         d2.get_world_component_id(0)))
+    dc.add_link(LinkSame(d1.world_component_ids[0],
+                         d2.world_component_ids[0]))
 
-    assert d2.get_pixel_component_id(0) not in d1.visible_components
-    np.testing.assert_array_equal(d1[d2.get_pixel_component_id(0)], [0])
+    assert d2.pixel_component_ids[0] not in d1.main_components
+    np.testing.assert_array_equal(d1[d2.pixel_component_ids[0]], [0])
 
 
 def test_add_binary_component():
@@ -680,7 +682,7 @@ def test_update_values_from_data():
     assert d1b in d1.components
     assert d2b not in d1.components
     assert d2c not in d1.components
-    assert [cid.label for cid in d1.visible_components] == ['b', 'c']
+    assert [cid.label for cid in d1.main_components] == ['b', 'c']
     assert d1.shape == (4,)
 
 
@@ -721,7 +723,7 @@ def test_update_values_from_data_order():
 
     d2.update_values_from_data(d1)
 
-    assert [cid.label for cid in d2.visible_components] == ['j', 'a', 'c', 'b', 'f']
+    assert [cid.label for cid in d2.main_components] == ['j', 'a', 'c', 'b', 'f']
 
 
 def test_find_component_id_with_cid():
@@ -833,3 +835,61 @@ def test_update_coords():
 
     # The link between the two world coordinates should be remove
     assert len(data_collection.links) == 4
+
+
+def test_compute_statistic_subset():
+
+    data = Data(x=list(range(10)))
+    result = data.compute_statistic('mean', data.id['x'], subset_state=data.id['x'] > 5)
+    assert_allclose(result, 7.5)
+
+    subset_state = SliceSubsetState(data, [slice(5)])
+    result = data.compute_statistic('mean', data.id['x'], subset_state=subset_state)
+    assert_allclose(result, 2.0)
+
+
+@pytest.mark.parametrize('shape', (100, (30, 10), (500, 1, 30)))
+def test_compute_statistic_chunks(shape):
+
+    # Make sure that when using chunks, the result is the same as without.
+
+    data = Data(x=np.random.random(shape))
+
+    axis = tuple(range(data.ndim - 1))
+    assert_allclose(data.compute_statistic('mean', data.id['x'], axis=axis),
+                    data.compute_statistic('mean', data.id['x'], axis=axis, n_chunk_max=10))
+
+
+def test_compute_statistic_random_subset():
+
+    data = Data(x=list(range(10)))
+
+    with NumpyRNGContext(12345):
+
+        result = data.compute_statistic('mean', data.id['x'], random_subset=5)
+        assert_allclose(result, 4.2)
+
+        result = data.compute_statistic('mean', data.id['x'], random_subset=5,
+                                        subset_state=MaskSubsetState([0, 1, 0, 1, 1, 1, 0, 1, 0, 1],
+                                                                     data.pixel_component_ids))
+        assert_allclose(result, 4.75)
+
+
+def test_compute_statistic_empty_subset():
+
+    data = Data(x=np.empty((30, 20, 40)))
+
+    # A default subset state should be empty
+    subset_state = SubsetState()
+
+    result = data.compute_statistic('mean', data.id['x'], subset_state=subset_state)
+    assert_equal(result, np.nan)
+
+    result = data.compute_statistic('maximum', data.id['x'], subset_state=subset_state, axis=1)
+    assert_equal(result, broadcast_to(np.nan, (30, 40)))
+
+    result = data.compute_statistic('median', data.id['x'], subset_state=subset_state, axis=(1, 2))
+    assert_equal(result, broadcast_to(np.nan, (30)))
+
+    result = data.compute_statistic('sum', data.id['x'], subset_state=subset_state, axis=(0, 1, 2))
+    assert_equal(result, np.nan)
