@@ -4,15 +4,11 @@ from __future__ import absolute_import, division, print_function
 
 import sys
 import optparse
+from importlib import import_module
 
 from glue import __version__
 from glue.logger import logger
 
-try:
-    from glue.utils.qt.decorators import die_on_error
-except ImportError:
-    from glue.utils.decorators import die_on_error
-
 
 def parse(argv):
     """ Parse argument list, check validity
@@ -60,6 +56,10 @@ def parse(argv):
                       help="Increase the vebosity level", default=False)
     parser.add_option('--no-maximized', action='store_true', dest='nomax',
                       help="Do not start Glue maximized", default=False)
+    parser.add_option('--startup', dest='startup', type='string',
+                      help="Startup actions to carry out", default='')
+    parser.add_option('--auto-merge', dest='auto_merge', action='store_true',
+                      help="Automatically merge any data passed on the command-line", default='')
 
     err_msg = verify(parser, argv)
     if err_msg:
@@ -95,21 +95,15 @@ def verify(parser, argv):
     return err_msg
 
 
-@die_on_error("Error restoring Glue session")
-def restore_session(gluefile):
-    """Load a .glu file and return a DataCollection, Hub tuple"""
-    from glue.app.qt import GlueApplication
-    return GlueApplication.restore_session(gluefile)
-
-
-@die_on_error("Error reading data file")
 def load_data_files(datafiles):
     """Load data files and return a list of datasets"""
-    from glue.core.data_factories import auto_data, load_data
+
+    from glue.core.data_factories import load_data
 
     datasets = []
     for df in datafiles:
-        datasets.append(load_data(df, auto_data))
+        datasets.append(load_data(df))
+
     return datasets
 
 
@@ -118,38 +112,66 @@ def run_tests():
     test()
 
 
-def start_glue(gluefile=None, config=None, datafiles=None, maximized=True):
+def start_glue(gluefile=None, config=None, datafiles=None, maximized=True,
+               startup_actions=None, auto_merge=False):
     """Run a glue session and exit
 
     Parameters
     ----------
     gluefile : str
         An optional ``.glu`` file to restore.
-
     config : str
         An optional configuration file to use.
-
     datafiles : str
         An optional list of data files to load.
-
     maximized : bool
         Maximize screen on startup. Otherwise, use default size.
-
+    auto_merge : bool, optional
+        Whether to automatically merge data passed in `datafiles` (default is `False`)
     """
+
     import glue
-    from glue.app.qt import GlueApplication
+
+    # Some Qt modules are picky in terms of being imported before the
+    # application is set up, so we import them here. We do it here rather
+    # than in get_qapp since in the past, including this code in get_qapp
+    # caused severe issues (e.g. segmentation faults) in plugin packages
+    # during testing.
+    try:
+        from qtpy import QtWebEngineWidgets  # noqa
+    except ImportError:  # Not all PyQt installations have this module
+        pass
+
+    from glue.utils.qt.decorators import die_on_error
+
+    from glue.utils.qt import get_qapp
+    app = get_qapp()
+
+    splash = get_splash()
+    splash.show()
 
     # Start off by loading plugins. We need to do this before restoring
     # the session or loading the configuration since these may use existing
     # plugins.
-    load_plugins()
+    load_plugins(splash=splash)
+
+    from glue.app.qt import GlueApplication
 
     datafiles = datafiles or []
 
     hub = None
 
+    from qtpy.QtCore import QTimer
+
+    timer = QTimer()
+    timer.setInterval(1000)
+    timer.setSingleShot(True)
+    timer.timeout.connect(splash.close)
+    timer.start()
+
     if gluefile is not None:
-        app = restore_session(gluefile)
+        with die_on_error("Error restoring Glue session"):
+            app = GlueApplication.restore_session(gluefile)
         return app.start()
 
     if config is not None:
@@ -158,43 +180,40 @@ def start_glue(gluefile=None, config=None, datafiles=None, maximized=True):
     data_collection = glue.core.DataCollection()
     hub = data_collection.hub
 
+    splash.set_progress(100)
+
     session = glue.core.Session(data_collection=data_collection, hub=hub)
-    ga = GlueApplication(session=session, maximized=maximized)
+    ga = GlueApplication(session=session)
 
     if datafiles:
-        datasets = load_data_files(datafiles)
-        ga.add_datasets(data_collection, datasets)
+        with die_on_error("Error reading data file"):
+            datasets = load_data_files(datafiles)
+        ga.add_datasets(data_collection, datasets, auto_merge=auto_merge)
 
-    # ga.show()
-    # splash.close()
-    # ga.raise_()
-    # QApplication.instance().processEvents()
-    return ga.start()
+    if startup_actions is not None:
+        for name in startup_actions:
+            ga.run_startup_action(name)
+
+    return ga.start(maximized=maximized)
 
 
-@die_on_error("Error running script")
 def execute_script(script):
     """ Run a python script and exit.
 
     Provides a way for people with pre-installed binaries to use
     the glue library
     """
-    with open(script) as fin:
-        exec(fin.read())
+    from glue.utils.qt.decorators import die_on_error
+    with die_on_error("Error running script"):
+        with open(script) as fin:
+            exec(fin.read())
     sys.exit(0)
 
 
 def get_splash():
     """Instantiate a splash screen"""
-    from qtpy import QtGui, QtWidgets
-    from qtpy.QtCore import Qt
-    import os
-
-    pth = os.path.join(os.path.dirname(__file__), 'logo.png')
-    pm = QtGui.QPixmap(pth)
-    splash = QtWidgets.QSplashScreen(pm, Qt.WindowStaysOnTopHint)
-    splash.show()
-
+    from glue.app.qt.splash_screen import QtSplashScreen
+    splash = QtSplashScreen()
     return splash
 
 
@@ -209,7 +228,11 @@ def main(argv=sys.argv):
 
     # Global keywords for Glue startup.
     kwargs = {'config': opt.config,
-              'maximized': not opt.nomax}
+              'maximized': not opt.nomax,
+              'auto_merge': opt.auto_merge}
+
+    if opt.startup:
+        kwargs['startup_actions'] = opt.startup.split(',')
 
     if opt.test:
         return run_tests()
@@ -235,8 +258,17 @@ def main(argv=sys.argv):
 _loaded_plugins = set()
 _installed_plugins = set()
 
+REQUIRED_PLUGINS = ['glue.plugins.tools.pv_slicer',
+                    'glue.plugins.coordinate_helpers',
+                    'glue.viewers.image',
+                    'glue.viewers.scatter',
+                    'glue.viewers.histogram',
+                    'glue.viewers.table',
+                    'glue.core.data_exporters',
+                    'glue.io.formats.fits']
 
-def load_plugins():
+
+def load_plugins(splash=None):
 
     # Search for plugins installed via entry_points. Basically, any package can
     # define plugins for glue, and needs to define an entry point using the
@@ -259,7 +291,9 @@ def load_plugins():
     from glue._plugin_helpers import iter_plugin_entry_points, PluginConfig
     config = PluginConfig.load()
 
-    for item in iter_plugin_entry_points():
+    n_plugins = len(list(iter_plugin_entry_points()))
+
+    for iplugin, item in enumerate(iter_plugin_entry_points()):
 
         if item.module_name not in _installed_plugins:
             _installed_plugins.add(item.name)
@@ -271,16 +305,39 @@ def load_plugins():
         if not config.plugins[item.name]:
             continue
 
+        # We don't use item.load() because that then checks requirements of all
+        # the imported packages, which can lead to errors like this one that
+        # don't really matter:
+        #
+        # Exception: (pytest 2.6.0 (/Users/tom/miniconda3/envs/py27/lib/python2.7/site-packages),
+        #             Requirement.parse('pytest>=2.8'), set(['astropy']))
+        #
+        # Just to be clear, this kind of error does indicate that there is an
+        # old version of a package in the environment, but this can confuse
+        # users as importing astropy directly would work (as setuptools then
+        # doesn't do a stringent test of dependency versions). Often this kind
+        # of error can occur if there is a conda version of a package and and
+        # older pip version.
+
         try:
-            function = item.load()
+            module = import_module(item.module_name)
+            function = getattr(module, item.attrs[0])
             function()
         except Exception as exc:
-            logger.info("Loading plugin {0} failed "
-                        "(Exception: {1})".format(item.name, exc))
+            # Here we check that some of the 'core' plugins load well and
+            # raise an actual exception if not.
+            if item.module_name in REQUIRED_PLUGINS:
+                raise
+            else:
+                logger.info("Loading plugin {0} failed "
+                            "(Exception: {1})".format(item.name, exc))
         else:
             logger.info("Loading plugin {0} succeeded".format(item.name))
             _loaded_plugins.add(item.module_name)
 
+        if splash is not None:
+            splash.set_progress(100. * iplugin / float(n_plugins))
+
     try:
         config.save()
     except Exception as e:
