@@ -1,5 +1,7 @@
 from __future__ import absolute_import, division, print_function
 
+import warnings
+
 import numpy as np
 from numpy.lib.stride_tricks import as_strided
 
@@ -13,7 +15,8 @@ from glue.external.six.moves import range
 __all__ = ['unique', 'shape_to_string', 'view_shape', 'stack_view',
            'coerce_numeric', 'check_sorted', 'broadcast_to', 'unbroadcast',
            'iterate_chunks', 'combine_slices', 'nanmean', 'nanmedian', 'nansum',
-           'nanmin', 'nanmax', 'format_minimal']
+           'nanmin', 'nanmax', 'format_minimal', 'compute_statistic',
+           'categorical_ndarray', 'index_lookup']
 
 
 def unbroadcast(array):
@@ -241,7 +244,7 @@ def iterate_chunks(shape, chunk_shape=None, n_max=None):
 
         end_index = [min(start_index[i] + chunk_shape[i], shape[i]) for i in range(ndim)]
 
-        slices = [slice(start_index[i], end_index[i]) for i in range(ndim)]
+        slices = tuple([slice(start_index[i], end_index[i]) for i in range(ndim)])
 
         yield slices
 
@@ -378,6 +381,8 @@ def format_minimal(values):
     ~30ms for a 1000 element array and 200ms for a 10000 element array. One
     could probably make a more efficient implementation but this is good enough
     for now for what we use it for.
+
+    Returns the optimal format as well as an array of formatted values.
     """
     values = np.asarray(values)
     if np.max(np.abs(values)) > 1e5 or np.min(np.diff(values)) < 1e-5:
@@ -390,3 +395,196 @@ def format_minimal(values):
         if len(strings) == len(set(strings)):
             break
     return fmt, strings
+
+
+PLAIN_FUNCTIONS = {'minimum': np.min,
+                   'maximum': np.max,
+                   'mean': np.mean,
+                   'median': np.median,
+                   'sum': np.sum,
+                   'percentile': np.percentile}
+
+NAN_FUNCTIONS = {'minimum': nanmin,
+                 'maximum': nanmax,
+                 'mean': nanmean,
+                 'median': nanmedian,
+                 'sum': nansum,
+                 'percentile': np.nanpercentile}
+
+
+def compute_statistic(statistic, data, mask=None, axis=None, finite=True,
+                      positive=False, percentile=None):
+    """
+    Compute a statistic for the data.
+
+    Parameters
+    ----------
+    statistic : {'minimum', 'maximum', 'mean', 'median', 'sum', 'percentile'}
+        The statistic to compute
+    data : `numpy.ndarray`
+        The data to compute the statistic for.
+    mask : `numpy.ndarray`
+        The mask to apply when computing the statistic.
+    axis : None or int or tuple of int
+        If specified, the axis/axes to compute the statistic over.
+    finite : bool, optional
+        Whether to include only finite values in the statistic. This should
+        be `True` to ignore NaN/Inf values
+    positive : bool, optional
+        Whether to include only (strictly) positive values in the statistic.
+        This is used for example when computing statistics of data shown in
+        log space.
+    percentile : float, optional
+        If ``statistic`` is ``'percentile'``, the ``percentile`` argument
+        should be given and specify the percentile to calculate in the
+        range [0:100]
+    """
+
+    data = np.asanyarray(data)
+    if mask is not None:
+        mask = np.asanyarray(mask, dtype=bool)
+
+    # NOTE: this function should not ever have to use glue-specific objects.
+    # The aim is to eventually use a fast C implementation of this function.
+
+    if statistic not in PLAIN_FUNCTIONS:
+        raise ValueError("Unrecognized statistic: {0}".format(statistic))
+
+    if (finite or positive or mask is not None) and data.dtype.kind != 'M':
+
+        keep = np.ones(data.shape, dtype=bool)
+
+        if finite:
+            keep &= np.isfinite(data)
+
+        if positive:
+            keep &= data > 0
+
+        if mask is not None:
+            keep &= mask
+
+        if axis is None:
+            data = data[keep]
+        else:
+            # We need to force a copy since we are editing the values and we
+            # might as well convert to float just in case
+            data = np.array(data, dtype=float)
+            data[~keep] = np.nan
+
+        function = NAN_FUNCTIONS[statistic]
+
+    else:
+
+        function = PLAIN_FUNCTIONS[statistic]
+
+    if data.size == 0:
+        return np.nan
+
+    if isinstance(axis, tuple) and len(axis) == 0:
+        return data
+
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", category=RuntimeWarning)
+        if statistic == 'percentile':
+            return function(data, percentile, axis=axis)
+        else:
+            return function(data, axis=axis)
+
+
+class categorical_ndarray(np.ndarray):
+    """
+    A Numpy array subclass that includes properties to find the categories and
+    unique integer codes for array values.
+    """
+
+    _jitter = None
+
+    def __new__(cls, value, dtype=None, copy=True, order=None, subok=False,
+                ndmin=0, categories=None):
+        result = np.array(value, dtype=dtype, copy=copy, order=order,
+                          subok=True, ndmin=ndmin).view(categorical_ndarray)
+        if categories is not None:
+            result.categories = categories
+        return result
+
+    def __array_finalize__(self, obj):
+        if isinstance(obj, categorical_ndarray):
+            self.categories = obj.categories
+
+    def _update_categories_and_codes(self):
+        if hasattr(self, '_categories'):
+            self._codes = index_lookup(self, self._categories)
+        else:
+            self._categories, self._codes = unique(self)
+            self._categories.setflags(write=False)
+            self._codes = self._codes.astype(float)
+            self._codes.setflags(write=False)
+
+    @property
+    def categories(self):
+        if not hasattr(self, '_categories'):
+            self._update_categories_and_codes()
+        return self._categories
+
+    @categories.setter
+    def categories(self, value):
+        self._categories = value
+
+    @property
+    def codes(self):
+        if not hasattr(self, '_codes'):
+            self._update_categories_and_codes()
+        if self._jitter is None:
+            return self._codes
+        else:
+            return self._codes + self._jitter
+
+    def jitter(self, method=None):
+        """
+        Jitter the codes.
+
+        Parameters
+        ----------
+        method : {None, 'uniform'}
+            If `None`, not jittering is done (or any jittering is undone).
+            If ``'uniform'``, the codes are randomized by a uniformly
+            distributed random variable.
+        """
+        if method is None:
+            self._jitter = None
+        elif method == 'uniform':
+            self._jitter = np.random.random(self.shape)
+            self._jitter -= 0.5
+        else:
+            raise ValueError("method should be None or 'uniform'")
+
+
+def index_lookup(data, items):
+    """
+    Lookup which index in items each data value is equal to
+
+    Parameters
+    ----------
+    data
+        An array-like object
+    items
+        Array-like of unique values
+
+    Returns
+    -------
+    array
+        If result[i] is finite, then data[i] = categories[result[i]]
+        Otherwise, data[i] is not in the categories list
+    """
+
+    # np.searchsorted doesn't work on mixed types in Python3
+
+    ndata, ncat = len(data), len(items)
+    data = pd.DataFrame({'data': data, 'row': np.arange(ndata)})
+    cats = pd.DataFrame({'items': items,
+                         'cat_row': np.arange(ncat)})
+
+    m = pd.merge(data, cats, left_on='data', right_on='items')
+    result = np.zeros(ndata, dtype=float) * np.nan
+    result[np.array(m.row)] = m.cat_row
+    return result
