@@ -6,29 +6,32 @@ import pytest
 import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib.patches import Circle
+from matplotlib.artist import Artist
 from numpy.testing import assert_allclose
+from matplotlib.backends.backend_agg import FigureCanvasAgg
 
-from glue.tests.helpers import requires_scipy
+from glue.tests.helpers import requires_scipy, requires_skimage
+from glue.utils.misc import DeferredMethod
 
 from ..matplotlib import (point_contour, fast_limits, all_artists, new_artists,
                           remove_artists, view_cascade, get_extent, color2rgb,
-                          defer_draw, freeze_margins)
+                          defer_draw, freeze_margins, datetime64_to_mpl,
+                          mpl_to_datetime64, color2hex)
 
 
 @requires_scipy
-class TestPointContour(object):
+@requires_skimage
+def test_point_contour():
+    data = np.array([[0, 0, 0, 0],
+                     [0, 2, 3, 0],
+                     [0, 4, 2, 0],
+                     [0, 0, 0, 0]])
+    xy = point_contour(2, 2, data)
+    x = np.array([2., 1., 0.5, 0.5, 1., 2., 2.5, 2.5, 2.])
+    y = np.array([2.5, 2.5, 2., 1., 0.5, 0.5, 1., 2., 2.5])
 
-    def test(self):
-        data = np.array([[0, 0, 0, 0],
-                         [0, 2, 3, 0],
-                         [0, 4, 2, 0],
-                         [0, 0, 0, 0]])
-        xy = point_contour(2, 2, data)
-        x = np.array([2., 2. + 1. / 3., 2., 2., 1, .5, 1, 1, 2])
-        y = np.array([2. / 3., 1., 2., 2., 2.5, 2., 1., 1., 2. / 3])
-
-        np.testing.assert_array_almost_equal(xy[:, 0], x)
-        np.testing.assert_array_almost_equal(xy[:, 1], y)
+    np.testing.assert_array_almost_equal(xy[:, 0], x)
+    np.testing.assert_array_almost_equal(xy[:, 1], y)
 
 
 def test_fast_limits_nans():
@@ -89,8 +92,40 @@ def test_defer_draw():
         fig = plt.figure()
         ax = fig.add_subplot(1, 1, 1)
         ax.plot([1, 2, 3], [4, 5, 6])
+        fig.canvas.draw()
+        return 3.5
 
-    draw_figure()
+    result = draw_figure()
+
+    # Make sure that the return value was passed through correctly
+    assert result == 3.5
+
+
+def test_defer_draw_exception(tmpdir):
+
+    # Regression test for a bug that meant that if an exception happened during
+    # drawing, the draw method was not restored correctly
+
+    # Make sure we start off with a clean draw method
+    assert not isinstance(FigureCanvasAgg.draw, DeferredMethod)
+
+    class ProblematicArtist(Artist):
+        def draw(self, *args, **kwargs):
+            raise ValueError('You shall not pass!')
+
+    @defer_draw
+    def draw_figure():
+        fig = plt.figure()
+        ax = fig.add_subplot(1, 1, 1)
+        ax.add_artist(ProblematicArtist())
+        fig.savefig(tmpdir.join('test.png').strpath)
+
+    with pytest.raises(ValueError) as exc:
+        result = draw_figure()
+    assert exc.value.args[0] == 'You shall not pass!'
+
+    # Make sure that draw is no longer a deferred method
+    assert not isinstance(FigureCanvasAgg.draw, DeferredMethod)
 
 
 @pytest.mark.parametrize(('color', 'rgb'),
@@ -99,6 +134,10 @@ def test_color2rgb(color, rgb):
     assert_allclose(color2rgb(color), rgb, atol=0.001)
 
 
+def test_color2hex():
+    assert color2hex('red') == '#ff0000'
+
+
 def test_freeze_margins():
 
     fig = plt.figure(figsize=(4, 4))
@@ -140,3 +179,10 @@ def test_freeze_margins():
     np.testing.assert_allclose(bbox.y0, 0.25)
     np.testing.assert_allclose(bbox.x1, 0.875)
     np.testing.assert_allclose(bbox.y1, 0.5)
+
+
+def test_mpl_datetime64():
+    # Make sure the mpl <-> datetime64 conversion round-trips
+    mpl1 = 719313
+    mpl2 = datetime64_to_mpl(mpl_to_datetime64(mpl1))
+    assert mpl1 == mpl2
