@@ -191,38 +191,38 @@ class TestHistogramViewer(object):
 
         assert_allclose(self.viewer.state.y_max, 2.4)
 
-        assert_allclose(self.viewer.layers[0].mpl_hist, [0, 1, 2, 1])
-        assert_allclose(self.viewer.layers[0].mpl_bins, [-5, -2.5, 0, 2.5, 5])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [0, 1, 2, 1])
+        assert_allclose(self.viewer.layers[0].state.histogram[0], [-5, -2.5, 0, 2.5, 5])
 
-        cid = self.data.visible_components[0]
+        cid = self.data.main_components[0]
         self.data_collection.new_subset_group('subset 1', cid < 2)
 
-        assert_allclose(self.viewer.layers[1].mpl_hist, [0, 1, 1, 0])
-        assert_allclose(self.viewer.layers[1].mpl_bins, [-5, -2.5, 0, 2.5, 5])
+        assert_allclose(self.viewer.layers[1].state.histogram[1], [0, 1, 1, 0])
+        assert_allclose(self.viewer.layers[1].state.histogram[0], [-5, -2.5, 0, 2.5, 5])
 
         viewer_state.normalize = True
 
         assert_allclose(self.viewer.state.y_max, 0.24)
-        assert_allclose(self.viewer.layers[0].mpl_hist, [0, 0.1, 0.2, 0.1])
-        assert_allclose(self.viewer.layers[0].mpl_bins, [-5, -2.5, 0, 2.5, 5])
-        assert_allclose(self.viewer.layers[1].mpl_hist, [0, 0.2, 0.2, 0])
-        assert_allclose(self.viewer.layers[1].mpl_bins, [-5, -2.5, 0, 2.5, 5])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [0, 0.1, 0.2, 0.1])
+        assert_allclose(self.viewer.layers[0].state.histogram[0], [-5, -2.5, 0, 2.5, 5])
+        assert_allclose(self.viewer.layers[1].state.histogram[1], [0, 0.2, 0.2, 0])
+        assert_allclose(self.viewer.layers[1].state.histogram[0], [-5, -2.5, 0, 2.5, 5])
 
         viewer_state.cumulative = True
 
         assert_allclose(self.viewer.state.y_max, 1.2)
-        assert_allclose(self.viewer.layers[0].mpl_hist, [0, 0.25, 0.75, 1.0])
-        assert_allclose(self.viewer.layers[0].mpl_bins, [-5, -2.5, 0, 2.5, 5])
-        assert_allclose(self.viewer.layers[1].mpl_hist, [0, 0.5, 1.0, 1.0])
-        assert_allclose(self.viewer.layers[1].mpl_bins, [-5, -2.5, 0, 2.5, 5])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [0, 0.25, 0.75, 1.0])
+        assert_allclose(self.viewer.layers[0].state.histogram[0], [-5, -2.5, 0, 2.5, 5])
+        assert_allclose(self.viewer.layers[1].state.histogram[1], [0, 0.5, 1.0, 1.0])
+        assert_allclose(self.viewer.layers[1].state.histogram[0], [-5, -2.5, 0, 2.5, 5])
 
         viewer_state.normalize = False
 
         assert_allclose(self.viewer.state.y_max, 4.8)
-        assert_allclose(self.viewer.layers[0].mpl_hist, [0, 1, 3, 4])
-        assert_allclose(self.viewer.layers[0].mpl_bins, [-5, -2.5, 0, 2.5, 5])
-        assert_allclose(self.viewer.layers[1].mpl_hist, [0, 1, 2, 2])
-        assert_allclose(self.viewer.layers[1].mpl_bins, [-5, -2.5, 0, 2.5, 5])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [0, 1, 3, 4])
+        assert_allclose(self.viewer.layers[0].state.histogram[0], [-5, -2.5, 0, 2.5, 5])
+        assert_allclose(self.viewer.layers[1].state.histogram[1], [0, 1, 2, 2])
+        assert_allclose(self.viewer.layers[1].state.histogram[0], [-5, -2.5, 0, 2.5, 5])
 
         viewer_state.cumulative = False
 
@@ -235,34 +235,34 @@ class TestHistogramViewer(object):
         assert xlabels == ['a', 'b', 'c']
 
         assert_allclose(self.viewer.state.y_max, 2.4)
-        assert_allclose(self.viewer.layers[0].mpl_hist, [2, 1, 1])
-        assert_allclose(self.viewer.layers[0].mpl_bins, [-0.5, 0.5, 1.5, 2.5])
-        assert_allclose(self.viewer.layers[1].mpl_hist, [1, 0, 1])
-        assert_allclose(self.viewer.layers[1].mpl_bins, [-0.5, 0.5, 1.5, 2.5])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [2, 1, 1])
+        assert_allclose(self.viewer.layers[0].state.histogram[0], [-0.5, 0.5, 1.5, 2.5])
+        assert_allclose(self.viewer.layers[1].state.histogram[1], [1, 0, 1])
+        assert_allclose(self.viewer.layers[1].state.histogram[0], [-0.5, 0.5, 1.5, 2.5])
 
         viewer_state.normalize = True
 
         assert_allclose(self.viewer.state.y_max, 0.6)
-        assert_allclose(self.viewer.layers[0].mpl_hist, [0.5, 0.25, 0.25])
-        assert_allclose(self.viewer.layers[0].mpl_bins, [-0.5, 0.5, 1.5, 2.5])
-        assert_allclose(self.viewer.layers[1].mpl_hist, [0.5, 0, 0.5])
-        assert_allclose(self.viewer.layers[1].mpl_bins, [-0.5, 0.5, 1.5, 2.5])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [0.5, 0.25, 0.25])
+        assert_allclose(self.viewer.layers[0].state.histogram[0], [-0.5, 0.5, 1.5, 2.5])
+        assert_allclose(self.viewer.layers[1].state.histogram[1], [0.5, 0, 0.5])
+        assert_allclose(self.viewer.layers[1].state.histogram[0], [-0.5, 0.5, 1.5, 2.5])
 
         viewer_state.cumulative = True
 
         assert_allclose(self.viewer.state.y_max, 1.2)
-        assert_allclose(self.viewer.layers[0].mpl_hist, [0.5, 0.75, 1])
-        assert_allclose(self.viewer.layers[0].mpl_bins, [-0.5, 0.5, 1.5, 2.5])
-        assert_allclose(self.viewer.layers[1].mpl_hist, [0.5, 0.5, 1])
-        assert_allclose(self.viewer.layers[1].mpl_bins, [-0.5, 0.5, 1.5, 2.5])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [0.5, 0.75, 1])
+        assert_allclose(self.viewer.layers[0].state.histogram[0], [-0.5, 0.5, 1.5, 2.5])
+        assert_allclose(self.viewer.layers[1].state.histogram[1], [0.5, 0.5, 1])
+        assert_allclose(self.viewer.layers[1].state.histogram[0], [-0.5, 0.5, 1.5, 2.5])
 
         viewer_state.normalize = False
 
         assert_allclose(self.viewer.state.y_max, 4.8)
-        assert_allclose(self.viewer.layers[0].mpl_hist, [2, 3, 4])
-        assert_allclose(self.viewer.layers[0].mpl_bins, [-0.5, 0.5, 1.5, 2.5])
-        assert_allclose(self.viewer.layers[1].mpl_hist, [1, 1, 2])
-        assert_allclose(self.viewer.layers[1].mpl_bins, [-0.5, 0.5, 1.5, 2.5])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [2, 3, 4])
+        assert_allclose(self.viewer.layers[0].state.histogram[0], [-0.5, 0.5, 1.5, 2.5])
+        assert_allclose(self.viewer.layers[1].state.histogram[1], [1, 1, 2])
+        assert_allclose(self.viewer.layers[1].state.histogram[0], [-0.5, 0.5, 1.5, 2.5])
 
         # TODO: add tests for log
 
@@ -287,8 +287,8 @@ class TestHistogramViewer(object):
 
         assert len(self.viewer.layers) == 2
 
-        assert_allclose(self.viewer.layers[0].mpl_hist, [0, 1, 2, 1])
-        assert_allclose(self.viewer.layers[1].mpl_hist, [0, 1, 2, 0])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [0, 1, 2, 1])
+        assert_allclose(self.viewer.layers[1].state.histogram[1], [0, 1, 2, 0])
 
         assert_allclose(self.data.subsets[0].to_mask(), [0, 1, 1, 1])
 
@@ -319,8 +319,8 @@ class TestHistogramViewer(object):
 
         assert len(self.viewer.layers) == 2
 
-        assert_allclose(self.viewer.layers[0].mpl_hist, [2, 1, 1])
-        assert_allclose(self.viewer.layers[1].mpl_hist, [2, 1, 0])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [2, 1, 1])
+        assert_allclose(self.viewer.layers[1].state.histogram[1], [2, 1, 0])
 
         assert_allclose(self.data.subsets[0].to_mask(), [1, 1, 0, 1])
 
@@ -379,13 +379,13 @@ class TestHistogramViewer(object):
 
         viewer_state.x_att = self.data.id['x']
 
-        cid = self.data.visible_components[0]
+        cid = self.data.main_components[0]
         self.data_collection.new_subset_group('subset 1', cid < 1)
 
-        cid = self.data.visible_components[0]
+        cid = self.data.main_components[0]
         self.data_collection.new_subset_group('subset 2', cid < 2)
 
-        cid = self.data.visible_components[0]
+        cid = self.data.main_components[0]
         self.data_collection.new_subset_group('subset 3', cid < 3)
 
         assert_allclose(self.viewer.state.y_min, 0)
@@ -412,26 +412,26 @@ class TestHistogramViewer(object):
         self.viewer.add_data(self.data)
 
         viewer_state.x_att = self.data.id['y']
-        viewer_state.hist_x_min = -10
+        viewer_state.hist_x_min = -10.1
         viewer_state.hist_x_max = +10
         viewer_state.hist_n_bin = 5
 
-        assert_allclose(self.viewer.layers[0].mpl_hist, [0, 0, 3, 1, 0])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [0, 0, 3, 1, 0])
 
         viewer_state.x_att = self.data.id['x']
-        viewer_state.hist_x_min = -10
+        viewer_state.hist_x_min = -10.1
         viewer_state.hist_x_max = +10
         viewer_state.hist_n_bin = 5
 
-        assert_allclose(self.viewer.layers[0].mpl_hist, [0, 0, 2, 2, 0])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [0, 0, 2, 2, 0])
 
         viewer_state.x_att = self.data.id['y']
 
-        assert_allclose(self.viewer.layers[0].mpl_hist, [0, 0, 3, 1, 0])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [0, 0, 3, 1, 0])
 
         viewer_state.x_att = self.data.id['x']
 
-        assert_allclose(self.viewer.layers[0].mpl_hist, [0, 0, 2, 2, 0])
+        assert_allclose(self.viewer.layers[0].state.histogram[1], [0, 0, 2, 2, 0])
 
     def test_component_replaced(self):
 
