@@ -9,11 +9,11 @@ from glue.utils import defer_draw, broadcast_to
 
 from glue.viewers.image.state import ImageLayerState, ImageSubsetLayerState
 from glue.viewers.image.python_export import python_export_image_layer, python_export_image_subset_layer
-from glue.viewers.image.pixel_selection_mode import PixelSubsetState
+from glue.viewers.image.pixel_selection_subset_state import PixelSubsetState
 from glue.viewers.matplotlib.layer_artist import MatplotlibLayerArtist
 from glue.core.exceptions import IncompatibleAttribute
 from glue.utils import color2rgb
-from glue.core import Data, HubListener
+from glue.core import BaseData, HubListener
 from glue.core.message import (ComponentsChangedMessage,
                                ExternallyDerivableComponentsChangedMessage,
                                PixelAlignedDataChangedMessage)
@@ -47,7 +47,7 @@ class BaseImageLayerArtist(MatplotlibLayerArtist, HubListener):
                                  filter=self._is_data_object)
 
     def _is_data_object(self, message):
-        if isinstance(self.layer, Data):
+        if isinstance(self.layer, BaseData):
             return message.sender is self.layer
         else:
             return message.sender is self.layer.data
@@ -103,7 +103,7 @@ class ImageLayerArtist(BaseImageLayerArtist):
 
     def get_image_shape(self):
 
-        if self._viewer_state.x_att is None or self._viewer_state.y_att is None:
+        if self._viewer_state.reference_data is None or self._viewer_state.x_att is None or self._viewer_state.y_att is None:
             return None
 
         x_axis = self._viewer_state.x_att.axis
@@ -247,7 +247,7 @@ class ImageSubsetArray(object):
             self.layer_artist.disable_incompatible_subset()
             return broadcast_to(np.nan, self.shape)
         else:
-            self.layer_artist.enable()
+            self.layer_artist.enable(redraw=False)
 
         r, g, b = color2rgb(self.layer_state.color)
         mask = np.dstack((r * mask, g * mask, b * mask, mask * .5))
@@ -306,10 +306,9 @@ class ImageSubsetLayerArtist(BaseImageLayerArtist):
             self._line_x.set_visible(False)
             self._line_y.set_visible(False)
         self.image_artist.invalidate_cache()
-        self.redraw()  # forces subset to be recomputed
 
     @defer_draw
-    def _update_visual_attributes(self):
+    def _update_visual_attributes(self, redraw=True):
 
         if not self.enabled:
             return
@@ -327,7 +326,8 @@ class ImageSubsetLayerArtist(BaseImageLayerArtist):
                 artist.set_alpha(self.state.alpha * 0.5)
             artist.set_zorder(self.state.zorder)
 
-        self.redraw()
+        if redraw:
+            self.redraw()
 
     def _update_image(self, force=False, **kwargs):
 
@@ -368,14 +368,14 @@ class ImageSubsetLayerArtist(BaseImageLayerArtist):
         super(ImageSubsetLayerArtist, self).remove()
         self.image_artist.invalidate_cache()
 
-    def enable(self):
+    def enable(self, redraw=True):
         super(ImageSubsetLayerArtist, self).enable()
         # We need to now ensure that image_artist, which may have been marked
         # as not being visible when the layer was cleared is made visible
         # again.
         if hasattr(self, 'image_artist'):
             self.image_artist.invalidate_cache()
-            self._update_visual_attributes()
+            self._update_visual_attributes(redraw=redraw)
 
     @defer_draw
     def update(self, *event):
