@@ -4,7 +4,7 @@ from collections import defaultdict
 
 import numpy as np
 
-from glue.core import Data
+from glue.core import BaseData, Coordinates
 from glue.config import colormaps
 from glue.viewers.matplotlib.state import (MatplotlibDataViewerState,
                                            MatplotlibLayerState,
@@ -79,12 +79,10 @@ class ImageViewerState(MatplotlibDataViewerState):
         self.ref_data_helper = ManualDataComboHelper(self, 'reference_data')
 
         self.xw_att_helper = ComponentIDComboHelper(self, 'x_att_world',
-                                                    numeric=False, categorical=False,
-                                                    world_coord=True)
+                                                    numeric=False, categorical=False)
 
         self.yw_att_helper = ComponentIDComboHelper(self, 'y_att_world',
-                                                    numeric=False, categorical=False,
-                                                    world_coord=True)
+                                                    numeric=False, categorical=False)
 
         self.add_callback('reference_data', self._reference_data_changed, priority=1000)
         self.add_callback('layers', self._layers_changed, priority=1000)
@@ -119,6 +117,13 @@ class ImageViewerState(MatplotlibDataViewerState):
             self.x_max = nx - 0.5
             self.y_min = -0.5
             self.y_max = ny - 0.5
+            # We need to adjust the limits in here to avoid triggering all
+            # the update events then changing the limits again.
+            self._adjust_limits_aspect()
+
+    @property
+    def _display_world(self):
+        return isinstance(getattr(self.reference_data, 'coords', None), Coordinates)
 
     def _reference_data_changed(self, *args):
         # This signal can get emitted if just the choices but not the actual
@@ -127,6 +132,16 @@ class ImageViewerState(MatplotlibDataViewerState):
         if self.reference_data is not getattr(self, '_last_reference_data', None):
             self._last_reference_data = self.reference_data
             with delay_callback(self, 'x_att_world', 'y_att_world', 'slices'):
+                if self._display_world:
+                    self.xw_att_helper.pixel_coord = False
+                    self.yw_att_helper.pixel_coord = False
+                    self.xw_att_helper.world_coord = True
+                    self.yw_att_helper.world_coord = True
+                else:
+                    self.xw_att_helper.pixel_coord = True
+                    self.yw_att_helper.pixel_coord = True
+                    self.xw_att_helper.world_coord = False
+                    self.yw_att_helper.world_coord = False
                 self._update_combo_att()
                 self._set_default_slices()
 
@@ -156,7 +171,7 @@ class ImageViewerState(MatplotlibDataViewerState):
         layer_state_by_data = defaultdict(list)
 
         for layer_state in self.layers:
-            if isinstance(layer_state.layer, Data):
+            if isinstance(layer_state.layer, BaseData):
                 layer_state_by_data[layer_state.layer].append(layer_state)
 
         for data, layer_states in layer_state_by_data.items():
@@ -196,23 +211,33 @@ class ImageViewerState(MatplotlibDataViewerState):
         # slicing with two pixel components that are the same.
         with delay_callback(self, 'x_att', 'y_att'):
             if self.x_att_world is not None:
-                index = self.reference_data.world_component_ids.index(self.x_att_world)
-                self.x_att = self.reference_data.pixel_component_ids[index]
+                if self._display_world:
+                    index = self.reference_data.world_component_ids.index(self.x_att_world)
+                    self.x_att = self.reference_data.pixel_component_ids[index]
+                else:
+                    self.x_att = self.x_att_world
             if self.y_att_world is not None:
-                index = self.reference_data.world_component_ids.index(self.y_att_world)
-                self.y_att = self.reference_data.pixel_component_ids[index]
+                if self._display_world:
+                    index = self.reference_data.world_component_ids.index(self.y_att_world)
+                    self.y_att = self.reference_data.pixel_component_ids[index]
+                else:
+                    self.y_att = self.y_att_world
 
     @defer_draw
     def _on_xatt_change(self, *args):
         if self.x_att is not None:
-            self.x_att_world = self.reference_data.world_component_ids[self.x_att.axis]
-        self.reset_limits()
+            if self._display_world:
+                self.x_att_world = self.reference_data.world_component_ids[self.x_att.axis]
+            else:
+                self.x_att_world = self.x_att
 
     @defer_draw
     def _on_yatt_change(self, *args):
         if self.y_att is not None:
-            self.y_att_world = self.reference_data.world_component_ids[self.y_att.axis]
-        self.reset_limits()
+            if self._display_world:
+                self.y_att_world = self.reference_data.world_component_ids[self.y_att.axis]
+            else:
+                self.y_att_world = self.y_att
 
     @defer_draw
     def _on_xatt_world_change(self, *args):
@@ -235,7 +260,7 @@ class ImageViewerState(MatplotlibDataViewerState):
     def _set_reference_data(self):
         if self.reference_data is None:
             for layer in self.layers:
-                if isinstance(layer.layer, Data):
+                if isinstance(layer.layer, BaseData):
                     self.reference_data = layer.layer
                     return
 
@@ -393,7 +418,7 @@ class BaseImageLayerState(MatplotlibLayerState):
         # to get the image to use.
 
         if self.layer.data is self.viewer_state.reference_data:
-            image = self._get_image(view=full_view)
+            image = self._get_image(view=tuple(full_view))
         else:
 
             # Second, we check whether the current data is linked pixel-wise with
@@ -412,7 +437,7 @@ class BaseImageLayerState(MatplotlibLayerState):
                 # doesn't make sense to show.
 
                 full_view = [full_view[idx] for idx in order]
-                image = self._get_image(view=full_view)
+                image = self._get_image(view=tuple(full_view))
 
                 if image.ndim != 2:
                     raise IncompatibleDataException()
@@ -502,7 +527,7 @@ class BaseImageLayerState(MatplotlibLayerState):
                 for icoord, coord in enumerate(coords):
                     keep[(coord < 0) | (coord >= self.layer.shape[icoord])] = False
                 coords = [coord[keep] for coord in coords]
-                image[keep] = self._get_image(view=coords)
+                image[keep] = self._get_image(view=tuple(coords))
 
                 # Finally convert array back to a 2D array
                 image = image.reshape(original_shape)
@@ -598,7 +623,7 @@ class ImageLayerState(BaseImageLayerState):
 
     attribute = DDSCProperty(docstring='The attribute shown in the layer')
     v_min = DDCProperty(docstring='The lower level shown')
-    v_max = DDCProperty(docstring='The upper leven shown')
+    v_max = DDCProperty(docstring='The upper level shown')
     percentile = DDSCProperty(docstring='The percentile value used to '
                                         'automatically calculate levels')
     contrast = DDCProperty(1, docstring='The contrast of the layer')
@@ -657,7 +682,7 @@ class ImageLayerState(BaseImageLayerState):
     def _update_attribute(self, *args):
         if self.layer is not None:
             self.attribute_att_helper.set_multiple_data([self.layer])
-            self.attribute = self.layer.visible_components[0]
+            self.attribute = self.layer.main_components[0]
 
     def _update_priority(self, name):
         if name == 'layer':
