@@ -1,5 +1,7 @@
 from __future__ import absolute_import, division, print_function
 
+from glue.utils import defer_draw, decorate_all_methods
+
 from glue.viewers.matplotlib.qt.data_viewer import MatplotlibDataViewer
 from glue.viewers.profile.qt.layer_style_editor import ProfileLayerStyleEditor
 from glue.viewers.profile.layer_artist import ProfileLayerArtist
@@ -8,11 +10,13 @@ from glue.viewers.profile.state import ProfileViewerState
 
 from glue.viewers.common.qt import toolbar_mode  # noqa
 from glue.viewers.profile.qt.profile_tools import ProfileAnalysisTool  # noqa
+from glue.viewers.profile.viewer import MatplotlibProfileMixin
 
 __all__ = ['ProfileViewer']
 
 
-class ProfileViewer(MatplotlibDataViewer):
+@decorate_all_methods(defer_draw)
+class ProfileViewer(MatplotlibProfileMixin, MatplotlibDataViewer):
 
     LABEL = '1D Profile'
     _layer_style_widget_cls = ProfileLayerStyleEditor
@@ -28,22 +32,5 @@ class ProfileViewer(MatplotlibDataViewer):
     tools = ['select:xrange', 'profile-analysis']
 
     def __init__(self, session, parent=None, state=None):
-        super(ProfileViewer, self).__init__(session, parent, state=state)
-        self.state.add_callback('x_att', self._update_axes)
-        self.state.add_callback('normalize', self._update_axes)
-
-    def _update_axes(self, *args):
-
-        if self.state.x_att is not None:
-            self.state.x_axislabel = self.state.x_att.label
-
-        if self.state.normalize:
-            self.state.y_axislabel = 'Normalized data values'
-        else:
-            self.state.y_axislabel = 'Data values'
-
-        self.axes.figure.canvas.draw()
-
-    def _roi_to_subset_state(self, roi):
-        x_comp = self.state.x_att.parent.get_component(self.state.x_att)
-        return x_comp.subset_from_roi(self.state.x_att, roi, coord='x')
+        MatplotlibDataViewer.__init__(self, session, parent=parent, state=state)
+        MatplotlibProfileMixin.setup_callbacks(self)
